
{***************************************************************************}
{                                                                           }
{     Form: 'ResNeuroBasicForm.dfm'                                         }
{     Unit: 'ResNeuroBasicFormFile.pas'                                     }
{     ( containing main menu enabling the various forms to be opened )      }
{                                                                           }
{     - belonging to:                                                       }
{       Project: 'ResNeuronum.dpr'                                          }
{       Program: 'ResNeuronum.exe'                                          }
{       ( simulating Neural Networks )                                      }
{                                                                           }
{     - written in Borland Delphi 2.0                                       }
{                                                                           }
{     Copyright (c) 1996-97 TU Ilmenau/ Germany                             }
{                                                                           }
{     Written by:                                                           }
{       Christian Heller                                                    }
{       Grenzgraben 04                                                      }
{       D-98714 Sttzerbach                                                 }
{       Thringen                                                           }
{       Deutschland/ Germany/ Allemagne                                     }
{       Tel.:   +49/ (0)36784/ 50471 (Euro-ISDN)                            }
{       or Tel: +49/ (0)36784/ 50690                                        }
{       Fax G3: +49/ (0)36784/ 50992 (Euro-ISDN)                            }
{       e-mail: Christian.Heller@rz.tu-ilmenau.de                           }
{       www:    http://www.rz.tu-ilmenau.de/~ai008/christian/enmain.htm     }
{       www:    http://www.rz.tu-ilmenau.de/~ai008/resneuronum/enmain.htm   }
{                                                                           }
{     ( all variables & methods in alphabetical order                       }
{     or as appearing in the unit header, respectively )                    }
{                                                                           }
{***************************************************************************}

unit ResNeuroBasicFormFile;

interface

uses
  Buttons,
  Classes,
  ComCtrls,
  Controls,
  Dialogs,
  ExtCtrls,
  Forms,
  Graphics,
  IniFiles,
  Menus,
  Messages,
  MPlayer,
  StdCtrls,
  SysUtils,
  Windows;

type
  TResNeuroBasicForm = class(TForm)
    AboutResNeuronum: TMenuItem;
    Applications: TMenuItem;
    BackPropagation: TMenuItem;
    BasicMediaPlayer: TMediaPlayer;
    BasicSpeedButtonApplication2: TSpeedButton;
    BasicSpeedButtonApplication3: TSpeedButton;
    BasicSpeedButtonBackPropagation: TSpeedButton;
    BasicSpeedButtonHopfield: TSpeedButton;
    BasicSpeedButtonKohonen: TSpeedButton;
    BasicSpeedButtonSpeechRecognition: TSpeedButton;
    BasicStatusBarHints: TStatusBar;
    BasicStatusBarStatus: TStatusBar;
    BevelLeft: TBevel;
    BevelTop: TBevel;
    BPBackPropagation: TMenuItem;
    Exit: TMenuItem;
    Files: TMenuItem;
    Help: TMenuItem;
    HelpLine: TMenuItem;
    Hopfield: TMenuItem;
    Kohonen: TMenuItem;
    LILaboratoryInstructions: TMenuItem;
    MainMenu: TMainMenu;
    Nets: TMenuItem;
    NNArtificialNeuralNetwork: TMenuItem;
    Options: TMenuItem;
    RNResNeuronum: TMenuItem;
    Settings: TMenuItem;
    SpeechRecognition: TMenuItem;
    SPSignalProcessing: TMenuItem;
    SRSpeechRecognition: TMenuItem;
    { Methods of BasicForm }
    procedure AboutResNeuronumClick(Sender: TObject);
    procedure BackPropagationClick(Sender: TObject);
    procedure BasicStatusBarStatusDrawPanel(StatusBar: TStatusBar;
      Panel: TStatusPanel; const Rect: TRect);
    procedure BPBackPropagationClick(Sender: TObject);
    procedure ExitClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure HopfieldClick(Sender: TObject);
    procedure KohonenClick(Sender: TObject);
    procedure LILaboratoryInstructionsClick(Sender: TObject);
    procedure LoadFromIni(IniName, IniSection: string);
    procedure NNArtificialNeuralNetworkClick(Sender: TObject);
    procedure RNResNeuronumClick(Sender: TObject);
    procedure SaveToIni(IniName, IniSection: string);
    procedure SettingsClick(Sender: TObject);
    procedure SpeechRecognitionClick(Sender: TObject);
    procedure SPSignalProcessingClick(Sender: TObject);
    procedure SRSpeechRecognitionClick(Sender: TObject);
  { Private-Deklarationen }
  private
    { StatusBar variables }
    BasicStatusImageIndex: Integer; // Index of current image
    { Methods }
    procedure BasicSetStatus(ImageIndex: Integer; Text: string); // Set status image and text
    procedure BasicStatusBarHintsShowHint(Sender: TObject); // Write hints onto panel
  { Public-Deklarationen }
  public
    { Settings Variables }
    SettBeenFunHymn: Boolean; // Thank you speech on/off
    SettExitApplHymn: Boolean; // Finishing speech on/off
    SettLogo: Boolean; // Title logo on/off
    SettLogoHymn: Boolean; // Title music on/off
  end;

  { Types for procedures importing NeuralNets from DLLs - explicitly }
  TCallBackPropagation = procedure;
  TCallHopfield = procedure;
  TCallKohonen = procedure;
  TCallSpeechRecognition = procedure;

  { Special Exception class for errors during loading DLL }
  EDLLLoadError = class(Exception);

var
  ResNeuroBasicForm: TResNeuroBasicForm;

implementation

uses
  { Self written units that are essential }
  ResNeuroAboutFormFile, // Form with information about authors
  ResNeuroLogoFormFile, // Form with TitleImage of application
  ResNeuroSettingsFormFile; // Form providing settings for application

{$R *.DFM}
{$R BASIC.RES}

procedure TResNeuroBasicForm.AboutResNeuronumClick(Sender: TObject);
begin
  { Set status }
  BasicSetStatus(2, 'Modales Formular "Info" aktiv');
  ResNeuroAboutForm := TResNeuroAboutForm.Create(self);
  try
    ResNeuroAboutForm.ShowModal;
  finally
    ResNeuroAboutForm.Free;
    { Set status }
    BasicSetStatus(0, 'Warte auf Eingabe');
  end;
end;

procedure TResNeuroBasicForm.BackPropagationClick(Sender: TObject);
var
  LibHandle: THandle;
  CallBackPropagation: TCallBackPropagation;
  ExeFilePath: string;
begin
  try
    { Set status in BasicForm }
    BasicSetStatus(3, 'Aktives Netz: FehlerRckfhrung');
    { Get Handle }
    LibHandle := LoadLibrary('BackPropagation.dll');
    if (LibHandle < 32) then begin
      MessageBeep(0);
      raise EDLLLoadError.Create('Kann DLL "BackPropagation.dll" fr Formular ' +
        '"Fehlerrckfhrungsnetz" nicht laden!');
    end;
    { Get address of dll }
    @CallBackPropagation := GetProcAddress(LibHandle, 'CallBackPropagation');
    { Call modal form BackPropForm }
    CallBackPropagation;
    { Free DLL }
    FreeLibrary(LibHandle);
    { Set status in BasicForm }
    BasicSetStatus(0, 'Warte auf Eingabe');
    with Application do begin
      { Get path of application }
      ExeFilePath := ExtractFilePath(ExeName);
      { Set hints and HelpFile back to those of BasicForm }
      OnHint := BasicStatusBarHintsShowHint;
      HelpFile := ExeFilePath + 'Help\ResNeuro.hlp';
    end;
  except
    on E: EDLLLoadError do begin
      { Set status in BasicForm }
      BasicSetStatus(0, 'Warte auf Eingabe');
      MessageBeep(0);
      MessageDlg(E.Message, mtInformation, [mbOk], 0);
    end;
  end;
end;

procedure TResNeuroBasicForm.BasicSetStatus(ImageIndex: Integer; Text: string);
var
  HelpRect: TRect;
begin
  { Set index of current image }
  BasicStatusImageIndex := ImageIndex;
  { Paint image }
  with BasicStatusBarStatus do begin
    with HelpRect do begin
      Left := 0;
      Top := 0;
      Right := Panels[0].Width;
      Bottom := Height;
    end;
    BasicStatusBarStatusDrawPanel(BasicStatusBarStatus, Panels[0], HelpRect);
    { Set text }
    Panels[1].Text := Text;
  end;
end;

procedure TResNeuroBasicForm.BasicStatusBarHintsShowHint(Sender: TObject);
begin
  BasicStatusBarHints.Panels[0].Text := Application.Hint;
end;

procedure TResNeuroBasicForm.BasicStatusBarStatusDrawPanel(StatusBar: TStatusBar;
  Panel: TStatusPanel; const Rect: TRect);
var
  Picture: TPicture;
begin
  Invalidate;
  { Overpaint old image }
  with StatusBar.Canvas do begin
    Brush.Color := clSilver;
    FillRect(Rect);
  end;
  { Paint new image }
  Picture := TPicture.Create;
  case BasicStatusImageIndex of
    0: Picture.Bitmap.LoadFromResourceName(HInstance, 'StatusGreen');
    1: Picture.Bitmap.LoadFromResourceName(HInstance, 'StatusYellow');
    2: Picture.Bitmap.LoadFromResourceName(HInstance, 'StatusRed');
    3: Picture.Bitmap.LoadFromResourceName(HInstance, 'StatusBackProp');
    4: Picture.Bitmap.LoadFromResourceName(HInstance, 'StatusSpeechRecog');
  end;
  StatusBar.Canvas.Draw(3, 3, TGraphic(Picture.Bitmap));
  Picture.Free;
end;

procedure TResNeuroBasicForm.BPBackPropagationClick(Sender: TObject);
var
  ExeFilePath: string;
begin
  with Application do begin
    ExeFilePath := ExtractFilePath(ExeName);
    HelpFile := ExeFilePath + 'Help\BackProp.hlp';
    HelpJump('BPBackPropagation');
  end;
end;

procedure TResNeuroBasicForm.ExitClick(Sender: TObject);
begin
  ResNeuroBasicForm.Close;
end;

procedure TResNeuroBasicForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  ExeFilePath: string;
begin
  { Set status }
  BasicSetStatus(1, 'Anwendung beenden');
  { Get path of application }
  ExeFilePath := ExtractFilePath(Application.ExeName);
  if (SettExitApplHymn = True) then begin
    with BasicMediaPlayer do begin
      DeviceType := dtWaveAudio;
      FileName := ExeFilePath + 'Sounds\ExitAppl.wav';
      try
        Open;
        Wait := true;
        Play;
        Stop;
      finally
        Close;
      end;
    end;
  end;
  { Finishing dialog }
  MessageBeep(0);
  if (MessageDlg('Programm "Res Neuronum" beenden? Alle aktiven Formulare werden geschlossen!',
    mtConfirmation, [mbYes, mbNo], 0) = mrYes) then CanClose := True
  else begin
    CanClose := False;
    { Set status }
    BasicSetStatus(0, 'Warte auf Eingabe');
  end;
end;

procedure TResNeuroBasicForm.FormCreate(Sender: TObject);
var
  ExeFilePath: string;
begin
  with Application do begin
    { Get path of Application }
    ExeFilePath := ExtractFilePath(ExeName);
    { Set HelpFile of BasicForm }
    HelpFile := ExeFilePath + 'Help\ResNeuro.hlp';
    { Set hints to StatusBar }
    OnHint := BasicStatusBarHintsShowHint;
  end;
  { Load settings from IniFile }
  LoadFromIni(ExeFilePath + 'ResNeuro.ini', 'Settings');
  { TitleMusic & LogoImage }
  if (SettLogoHymn = True) then begin
    with BasicMediaPlayer do begin
      DeviceType := dtWaveAudio;
      FileName := ExeFilePath + 'Sounds\LogoHymn.wav';
      try
        Open;
        if (SettLogo = True) then begin
          { Play TitleMusic }
          Play;
          { Display LogoImage }
          try
            ResNeuroLogoForm := TResNeuroLogoForm.Create(Self);
            ResNeuroLogoForm.ShowModal;
          finally
            ResNeuroLogoForm.Free;
          end;
          Stop;
        end
        else begin
          Wait := True;
          Play;
          Stop;
        end;
      finally
        Close;
      end;
    end;
  end
  else if (SettLogo = True) then begin
    { Display LogoImage }
    try
      ResNeuroLogoForm := TResNeuroLogoForm.Create(Self);
      ResNeuroLogoForm.ShowModal;
    finally
      ResNeuroLogoForm.Free;
    end;
  end;
  { Set size of basic form }
  with ResNeuroBasicForm do begin
    Left := 0;
    Width := Screen.Width;
    Top := 0;
  end;
  BasicStatusBarStatus.Width := (Screen.Width - 220);
  BasicStatusBarHints.Width := (Screen.Width - 220);
  { Set status }
  BasicSetStatus(0, 'Warte auf Eingabe');
end;

procedure TResNeuroBasicForm.FormDestroy(Sender: TObject);
var
  ExeFilePath: string;
begin
  { Get path of Application }
  ExeFilePath := ExtractFilePath(Application.ExeName);
  { Play music }
  if (SettBeenFunHymn = True) then begin
    with BasicMediaPlayer do begin
      DeviceType := dtWaveAudio;
      FileName := ExeFilePath + 'Sounds\BeenFun.wav';
      try
        Open;
        Wait := True;
        Play;
        Stop;
      finally
        Close;
      end;
    end;
  end;
  { Write settings into IniFile }
  SaveToIni(ExeFilePath + 'ResNeuro.ini', 'Settings');
end;

procedure TResNeuroBasicForm.HopfieldClick(Sender: TObject);
var
  LibHandle: THandle;
  CallHopfield: TCallHopfield;
  ExeFilePath: string;
begin
  try
    LibHandle := LoadLibrary('Hopfield.dll');
    if (LibHandle < 32) then begin
      MessageBeep(0);
      raise EDLLLoadError.Create('Kann DLL "Hopfield.dll" fr Formular "Hopfield" nicht laden!');
    end;
    { Get address of dll }
    @CallHopfield := GetProcAddress(LibHandle, 'CallHopfield');
    { Set status in BasicForm }
    BasicSetStatus(3, 'Aktives Netz: Hopfield');
    { Call modal form BackPropForm }
    CallHopfield;
    { Free DLL }
    FreeLibrary(LibHandle);
    { Set status in BasicForm }
    BasicSetStatus(0, 'Warte auf Eingabe');
    with Application do begin
      { Get path of application }
      ExeFilePath := ExtractFilePath(ExeName);
      { Set hints and HelpFile back to those of BasicForm }
      OnHint := BasicStatusBarHintsShowHint;
      HelpFile := ExeFilePath + 'Help\ResNeuro.hlp';
    end;
  except
    on E: EDLLLoadError do begin
      MessageBeep(0);
      MessageDlg(E.Message, mtInformation, [mbOk], 0);
    end;
  end;
end;

procedure TResNeuroBasicForm.KohonenClick(Sender: TObject);
var
  LibHandle: THandle;
  CallKohonen: TCallKohonen;
  ExeFilePath: string;
begin
  try
    LibHandle := LoadLibrary('Kohonen.dll');
    if (LibHandle < 32) then begin
      MessageBeep(0);
      raise EDLLLoadError.Create('Kann DLL "Kohonen.dll" fr Formular "Kohonennetz" nicht laden!');
    end;
    { Get address of dll }
    @CallKohonen := GetProcAddress(LibHandle, 'CallKohonen');
    { Set status in BasicForm }
    BasicSetStatus(3, 'Aktives Netz: Kohonen');
    { Call modal form BackPropForm }
    CallKohonen;
    { Free DLL }
    FreeLibrary(LibHandle);
    { Set status in BasicForm }
    BasicSetStatus(0, 'Warte auf Eingabe');
    with Application do begin
      { Get path of application }
      ExeFilePath := ExtractFilePath(ExeName);
      { Set hints and HelpFile back to those of BasicForm }
      OnHint := BasicStatusBarHintsShowHint;
      HelpFile := ExeFilePath + 'Help\ResNeuro.hlp';
    end;
  except
    on E: EDLLLoadError do begin
      MessageBeep(0);
      MessageDlg(E.Message, mtInformation, [mbOk], 0);
    end;
  end;
end;

procedure TResNeuroBasicForm.LILaboratoryInstructionsClick(Sender: TObject);
var
  ExeFilePath: string;
begin
  ExeFilePath := ExtractFilePath(Application.ExeName);
  Application.HelpFile := ExeFilePath + 'Help\LabInstr.hlp';
  Application.HelpJump('LILaboratoryInstructions');
end;

procedure TResNeuroBasicForm.LoadFromIni(IniName, IniSection: string);
var
  IniFile: TIniFile;
  TempLogo: string;
  TempLogoHymn: string;
  TempExitApplHymn: string;
  TempBeenFunHymn: string;
begin
  IniFile := TIniFile.Create(IniName);
  try
    TempLogo := IniFile.ReadString(IniSection, 'Logo', 'True');
    if TempLogo = 'True' then SettLogo := True
    else SettLogo := False;
    TempLogoHymn := IniFile.ReadString(IniSection, 'LogoHymn', 'True');
    if TempLogoHymn = 'True' then SettLogoHymn := True
    else SettLogoHymn := False;
    TempExitApplHymn := IniFile.ReadString(IniSection, 'ExitApplHymn', 'True');
    if TempExitApplHymn = 'True' then SettExitApplHymn := True
    else SettExitApplHymn := False;
    TempBeenFunHymn := IniFile.ReadString(IniSection, 'BeenFunHymn', 'True');
    if TempBeenFunHymn = 'True' then SettBeenFunHymn := True
    else SettBeenFunHymn := False;
  finally
    IniFile.Free;
  end;
end;

procedure TResNeuroBasicForm.NNArtificialNeuralNetworkClick(Sender: TObject);
var
  ExeFilePath: string;
begin
  ExeFilePath := ExtractFilePath(Application.ExeName);
  Application.HelpFile := ExeFilePath + 'Help\NeuroNet.hlp';
  Application.HelpJump('NNArtificialNeuralNetwork');
end;

procedure TResNeuroBasicForm.RNResNeuronumClick(Sender: TObject);
var
  ExeFilePath: string;
begin
  ExeFilePath := ExtractFilePath(Application.ExeName);
  Application.HelpFile := ExeFilePath + 'Help\ResNeuro.hlp';
  Application.HelpJump('RNResNeuronum');
end;

procedure TResNeuroBasicForm.SaveToIni(IniName, IniSection: string);
var
  IniFile: TIniFile;
  TempLogo: string;
  TempLogoHymn: string;
  TempExitApplHymn: string;
  TempBeenFunHymn: string;
begin
  IniFile := TIniFile.Create(IniName);
  try
    if SettLogo = True then TempLogo := 'True'
    else TempLogo := 'False';
    IniFile.WriteString(IniSection, 'Logo', TempLogo);
    if SettLogoHymn = True then TempLogoHymn := 'True'
    else TempLogoHymn := 'False';
    IniFile.WriteString(IniSection, 'LogoHymn', TempLogoHymn);
    if SettExitApplHymn = True then TempExitApplHymn := 'True'
    else TempExitApplHymn := 'False';
    IniFile.WriteString(IniSection, 'ExitApplHymn', TempExitApplHymn);
    if SettBeenFunHymn = True then TempBeenFunHymn := 'True'
    else TempBeenFunHymn := 'False';
    IniFile.WriteString(IniSection, 'BeenFunHymn', TempBeenFunHymn);
  finally
    IniFile.Free;
  end;
end;

procedure TResNeuroBasicForm.SettingsClick(Sender: TObject);
begin
  { Set status }
  BasicSetStatus(2, 'Modales Formular "Einstellungen" aktiv');
  ResNeuroSettingsForm := TResNeuroSettingsForm.Create(self);
  try
    ResNeuroSettingsForm.ShowModal;
  finally
    ResNeuroSettingsForm.Free;
    { Set status }
    BasicSetStatus(0, 'Warte auf Eingabe');
  end;
end;

procedure TResNeuroBasicForm.SpeechRecognitionClick(Sender: TObject);
var
  LibHandle: THandle;
  CallSpeechRecognition: TCallSpeechRecognition;
  ExeFilePath: string;
begin
  try
    { Set status in BasicForm }
    BasicSetStatus(4, 'Aktive Anwendung: SprachErkennung');
    { Get Handle }
    LibHandle := LoadLibrary('SpeechRecognition.dll');
    if (LibHandle < 32) then begin
      MessageBeep(0);
      raise EDLLLoadError.Create('Kann DLL "SpeechRecognition.dll" fr Formular "Spracherkennung" nicht laden!');
    end;
    { Get address of dll }
    @CallSpeechRecognition := GetProcAddress(LibHandle, 'CallSpeechRecognition');
    { Call modal form BackPropForm }
    CallSpeechRecognition;
    { Free DLL }
    FreeLibrary(LibHandle);
    { Set status in BasicForm }
    BasicSetStatus(0, 'Warte auf Eingabe');
    with Application do begin
      { Get path of application }
      ExeFilePath := ExtractFilePath(ExeName);
      { Set hints and HelpFile back to those of BasicForm }
      OnHint := BasicStatusBarHintsShowHint;
      HelpFile := ExeFilePath + 'Help\ResNeuro.hlp';
    end;
  except
    on E: EDLLLoadError do begin
      { Set status in BasicForm }
      BasicSetStatus(0, 'Warte auf Eingabe');
      MessageBeep(0);
      MessageDlg(E.Message, mtInformation, [mbOk], 0);
    end;
  end;
end;

procedure TResNeuroBasicForm.SPSignalProcessingClick(Sender: TObject);
var
  ExeFilePath: string;
begin
  ExeFilePath := ExtractFilePath(Application.ExeName);
  Application.HelpFile := ExeFilePath + 'Help\SignProc.hlp';
  Application.HelpJump('SPSignalProcessing');
end;

procedure TResNeuroBasicForm.SRSpeechRecognitionClick(Sender: TObject);
var
  ExeFilePath: string;
begin
  ExeFilePath := ExtractFilePath(Application.ExeName);
  Application.HelpFile := ExeFilePath + 'Help\SpeRecog.hlp';
  Application.HelpJump('SRSpeechRecognition');
end;

end.
