
{***************************************************************************}
{                                                                           }
{     Unit: 'SpeechRecogDynArrayFile.pas'                                   }
{     ( containing dynamic array for storing different values )             }
{                                                                           }
{     - belonging to:                                                       }
{       Project: 'SpeechRecognition.dpr'                                    }
{       DLL:     'SpeechRecognition.dll'                                    }
{       Program: 'ResNeuronum.exe'                                          }
{       ( simulating Neural Networks )                                      }
{                                                                           }
{     - written in Borland Delphi 2.0                                       }
{                                                                           }
{     Copyright (c) 1996-97 TU Ilmenau/ Germany                             }
{                                                                           }
{     Written by:                                                           }
{       Andreas Heyn                                                        }
{       D- Erfurt                                                           }
{       Thringen                                                           }
{       Deutschland/ Germany/ Allemagne                                     }
{                                                                           }
{       &                                                                   }
{                                                                           }
{       Christian Heller                                                    }
{       Grenzgraben 04                                                      }
{       D-98714 Sttzerbach                                                 }
{       Thringen                                                           }
{       Deutschland/ Germany/ Allemagne                                     }
{       Tel.:   +49/ (0)36784/ 50471 (Euro-ISDN)                            }
{       or Tel: +49/ (0)36784/ 50690                                        }
{       Fax G3: +49/ (0)36784/ 50992 (Euro-ISDN)                            }
{       e-mail: Christian.Heller@rz.tu-ilmenau.de                           }
{       www:    http://www.rz.tu-ilmenau.de/~ai008/christian/enmain.htm     }
{       www:    http://www.rz.tu-ilmenau.de/~ai008/resneuronum/enmain.htm   }
{                                                                           }
{     ( all variables & methods in alphabetical order                       }
{     or as appearing in the unit header, respectively )                    }
{                                                                           }
{***************************************************************************}

unit SpeechRecogDynArrayFile;

{$R-}

interface

uses Windows, SysUtils;

type
  { Array of String }
  TItemType = string;
  PDynArray = ^TDynArray;
  TDynArray = array[0..0] of TItemType;
  TDynamicArray = class
  private
    ArrayPtr: PDynArray;
    { Property data }
    FCapacity: Integer;
    FCount: Integer;
    { Property methods }
    function Get(Index: Integer): TItemType;
    procedure Put(Index: Integer; const Value: TItemType);
    procedure SetCapacity(Value: Integer);
  public
    constructor Create(cCount: Integer); virtual;
    destructor Destroy; override;
    function Add(const Value: TItemType): Integer; virtual;
    procedure Delete(Index: Integer); virtual;
    procedure DeleteAll; virtual;
    procedure Insert(Index: Integer; const Value: TItemType); virtual;
    property Capacity: Integer read FCapacity write SetCapacity;
    property Count: Integer read FCount;
    property Items[index: Integer]: TItemType read Get write Put; default;
  end;

  { Array of Integer }
  PIntArray = ^TIntArray;
  TIntArray = array[0..0] of Integer;
  TIntegerArray = class
  private
    ArrayPtr: PIntArray;
    { Property data }
    FCapacity: Integer;
    FCount: Integer;
    { Property methods }
    function Get(Index: Integer): Integer;
    procedure Put(Index: Integer; const Value: Integer);
    procedure SetCapacity(Value: Integer);
  public
    constructor Create(cCount: Integer); virtual;
    destructor Destroy; override;
    function Add(const Value: Integer): Integer; virtual;
    procedure Delete(Index: Integer); virtual;
    procedure DeleteAll; virtual;
    procedure Insert(Index: Integer; const Value: Integer); virtual;
    property Capacity: Integer read FCapacity write SetCapacity;
    property Count: Integer read FCount;
    property Items[index: Integer]: Integer read Get write Put; default;
  end;

  { Array of Double }
  PDoubArray = ^TDoubArray;
  TDoubArray = array[0..0] of Double;
  TDoubleArray = class
  private
    ArrayPtr: PDoubArray;
    { Property data }
    FCapacity: Integer;
    FCount: Integer;
    { Property methods }
    function Get(Index: Integer): Double;
    procedure Put(Index: Integer; const Value: Double);
    procedure SetCapacity(Value: Integer);
  public
    constructor Create(cCount: Integer); virtual;
    destructor Destroy; override;
    function Add(const Value: Double): Integer; virtual;
    procedure Delete(Index: Integer); virtual;
    procedure DeleteAll; virtual;
    procedure Insert(Index: Integer; const Value: Double); virtual;
    property Capacity: Integer read FCapacity write SetCapacity;
    property Count: Integer read FCount;
    property Items[index: Integer]: Double read Get write Put; default;
  end;

implementation

{ Methodes of TDynamicArray }

constructor TDynamicArray.Create(cCount: Integer);
begin
  inherited Create;
  if (cCount = 0) then Exit;
  GetMem(ArrayPtr, (cCount * SizeOf(TItemType)));
  if (ArrayPtr = nil) then OutOfMemoryError;
  FCount := cCount;
  FCapacity := cCount;
end;

destructor TDynamicArray.Destroy;
begin
  if (ArrayPtr <> nil) then FreeMem(ArrayPtr, 0);
  inherited Destroy;
end;

function TDynamicArray.Add(const Value: TItemType): Integer;
begin
  Result := FCount;
  if (FCount = FCapacity) then begin
    Inc(FCapacity, 8);
    ReallocMem(ArrayPtr, (FCapacity * SizeOf(TItemType)));
  end;
  ArrayPtr^[FCount] := Value;
  Inc(FCount);
end;

procedure TDynamicArray.Delete(Index: Integer);
begin
  if ((Index < 0) or (Index >= FCount)) then
    raise ERangeError.Create('Bereichsberschreitung');
  Dec(FCount);
  System.Move(ArrayPtr^[Index + 1], ArrayPtr^[Index],
    (FCount - Index) * SizeOf(TItemType));
end;

procedure TDynamicArray.DeleteAll;
begin
  ReallocMem(ArrayPtr, 0);
  FCapacity := 0;
  FCount := 0;
end;

function TDynamicArray.Get(Index: Integer): TItemType;
begin
  if ((Index < 0) or (Index >= FCount)) then
    raise ERangeError.Create('Bereichsberschreitung!');
  Result := ArrayPtr^[Index];
end;

procedure TDynamicArray.Insert(Index: Integer; const Value: TItemType);
begin
  if ((Index < 0) or (Index > FCount)) then
    raise ERangeError.Create('Bereichsberschreitung');
  if (FCount = FCapacity) then begin
    Inc(FCapacity, 8);
    ReallocMem(ArrayPtr, (FCapacity * SizeOf(TItemType)));
  end;
  if (Index < FCount) then
    System.Move(ArrayPtr^[Index], ArrayPtr^[Index + 1],
      (FCount - Index) * SizeOf(TItemType));
  ArrayPtr^[Index] := Value;
  Inc(FCount);
end;

procedure TDynamicArray.Put(Index: Integer; const Value: TItemType);
begin
  if ((Index < 0) or (Index >= FCount)) then
    raise ERangeError.Create('Bereichsberschreitung');
  ArrayPtr^[Index] := Value;
end;

procedure TDynamicArray.SetCapacity(Value: Integer);
begin
  if (Value < FCount) then
    raise ERangeError.Create('Bereichsberschreitung');
  if (Value <> FCapacity) then begin
    ReallocMem(ArrayPtr, (Value * SizeOf(TItemType)));
    FCapacity := Value;
  end;
end;

{ Methodes of TIntegerArray }

constructor TIntegerArray.Create(cCount: Integer);
begin
  inherited Create;
  if (cCount = 0) then Exit;
  GetMem(ArrayPtr, (cCount * SizeOf(Integer)));
  if (ArrayPtr = nil) then OutOfMemoryError;
  FCount := cCount;
  FCapacity := cCount;
end;

destructor TIntegerArray.Destroy;
begin
  if (ArrayPtr <> nil) then FreeMem(ArrayPtr, 0);
  inherited Destroy;
end;

function TIntegerArray.Add(const Value: Integer): Integer;
begin
  Result := FCount;
  if (FCount = FCapacity) then begin
    Inc(FCapacity, 8);
    ReallocMem(ArrayPtr, (FCapacity * SizeOf(Integer)));
  end;
  ArrayPtr^[FCount] := Value;
  Inc(FCount);
end;

procedure TIntegerArray.Delete(Index: Integer);
begin
  if ((Index < 0) or (Index >= FCount)) then
    raise ERangeError.Create('Bereichsberschreitung');
  Dec(FCount);
  System.Move(ArrayPtr^[Index + 1], ArrayPtr^[Index],
    (FCount - Index) * SizeOf(Integer));
end;

procedure TIntegerArray.DeleteAll;
begin
  ReallocMem(ArrayPtr, 0);
  FCapacity := 0;
  FCount := 0;
end;

function TIntegerArray.Get(Index: Integer): Integer;
begin
  if ((Index < 0) or (Index >= FCount)) then
    raise ERangeError.Create('Bereichsberschreitung!');
  Result := ArrayPtr^[Index];
end;

procedure TIntegerArray.Insert(Index: Integer; const Value: Integer);
begin
  if ((Index < 0) or (Index > FCount)) then
    raise ERangeError.Create('Bereichsberschreitung');
  if (FCount = FCapacity) then begin
    Inc(FCapacity, 8);
    ReallocMem(ArrayPtr, (FCapacity * SizeOf(Integer)));
  end;
  if (Index < FCount) then
    System.Move(ArrayPtr^[Index], ArrayPtr^[Index + 1],
      (FCount - Index) * SizeOf(Integer));
  ArrayPtr^[Index] := Value;
  Inc(FCount);
end;

procedure TIntegerArray.Put(Index: Integer; const Value: Integer);
begin
  if ((Index < 0) or (Index >= FCount)) then
    raise ERangeError.Create('Bereichsberschreitung');
  ArrayPtr^[Index] := Value;
end;

procedure TIntegerArray.SetCapacity(Value: Integer);
begin
  if (Value < FCount) then
    raise ERangeError.Create('Bereichsberschreitung');
  if (Value <> FCapacity) then begin
    ReallocMem(ArrayPtr, (Value * SizeOf(Integer)));
    FCapacity := Value;
  end;
end;

{ Methodes of TDoubleArray }

constructor TDoubleArray.Create(cCount: Integer);
begin
  inherited Create;
  if (cCount = 0) then Exit;
  GetMem(ArrayPtr, (cCount * SizeOf(Double)));
  if (ArrayPtr = nil) then OutOfMemoryError;
  FCount := cCount;
  FCapacity := cCount;
end;

destructor TDoubleArray.Destroy;
begin
  if (ArrayPtr <> nil) then FreeMem(ArrayPtr, 0);
  inherited Destroy;
end;

function TDoubleArray.Add(const Value: Double): Integer;
begin
  Result := FCount;
  if (FCount = FCapacity) then begin
    Inc(FCapacity, 8);
    ReallocMem(ArrayPtr, (FCapacity * SizeOf(Double)));
  end;
  ArrayPtr^[FCount] := Value;
  Inc(FCount);
end;

procedure TDoubleArray.Delete(Index: Integer);
begin
  if ((Index < 0) or (Index >= FCount)) then
    raise ERangeError.Create('Bereichsberschreitung');
  Dec(FCount);
  System.Move(ArrayPtr^[Index + 1], ArrayPtr^[Index],
    (FCount - Index) * SizeOf(Double));
end;

procedure TDoubleArray.DeleteAll;
begin
  ReallocMem(ArrayPtr, 0);
  FCapacity := 0;
  FCount := 0;
end;

function TDoubleArray.Get(Index: Integer): Double;
begin
  if ((Index < 0) or (Index >= FCount)) then
    raise ERangeError.Create('Bereichsberschreitung!');
  Result := ArrayPtr^[Index];
end;

procedure TDoubleArray.Insert(Index: Integer; const Value: Double);
begin
  if ((Index < 0) or (Index > FCount)) then
    raise ERangeError.Create('Bereichsberschreitung');
  if (FCount = FCapacity) then begin
    Inc(FCapacity, 8);
    ReallocMem(ArrayPtr, (FCapacity * SizeOf(Double)));
  end;
  if (Index < FCount) then
    System.Move(ArrayPtr^[Index], ArrayPtr^[Index + 1],
      (FCount - Index) * SizeOf(Double));
  ArrayPtr^[Index] := Value;
  Inc(FCount);
end;

procedure TDoubleArray.Put(Index: Integer; const Value: Double);
begin
  if ((Index < 0) or (Index >= FCount)) then
    raise ERangeError.Create('Bereichsberschreitung');
  ArrayPtr^[Index] := Value;
end;

procedure TDoubleArray.SetCapacity(Value: Integer);
begin
  if (Value < FCount) then
    raise ERangeError.Create('Bereichsberschreitung');
  if (Value <> FCapacity) then begin
    ReallocMem(ArrayPtr, (Value * SizeOf(Double)));
    FCapacity := Value;
  end;
end;

end.
