
{***************************************************************************}
{                                                                           }
{     Form: 'SpeechRecogForm.dfm'                                           }
{     Unit: 'SpeechRecogFormFile.pas'                                       }
{     ( containing audio application for Neural Networks )                  }
{                                                                           }
{     - belonging to:                                                       }
{       Project: 'SpeechRecognition.dpr'                                    }
{       DLL:     'SpeechRecognition.dll'                                    }
{       Program: 'ResNeuronum.exe'                                          }
{       ( simulating Neural Networks )                                      }
{                                                                           }
{     - written in Borland Delphi 2.0                                       }
{                                                                           }
{     Copyright (c) 1996-97 TU Ilmenau/ Germany                             }
{                                                                           }
{     Written by:                                                           }
{       Christian Heller                                                    }
{       Grenzgraben 04                                                      }
{       D-98714 Sttzerbach                                                 }
{       Thringen                                                           }
{       Deutschland/ Germany/ Allemagne                                     }
{       Tel.:   +49/ (0)36784/ 50471 (Euro-ISDN)                            }
{       or Tel: +49/ (0)36784/ 50690                                        }
{       Fax G3: +49/ (0)36784/ 50992 (Euro-ISDN)                            }
{       e-mail: Christian.Heller@rz.tu-ilmenau.de                           }
{       www:    http://www.rz.tu-ilmenau.de/~ai008/christian/enmain.htm     }
{       www:    http://www.rz.tu-ilmenau.de/~ai008/resneuronum/enmain.htm   }
{                                                                           }
{     ( all variables & methods in alphabetical order                       }
{     or as appearing in the unit header, respectively )                    }
{                                                                           }
{***************************************************************************}

unit SpeechRecogFormFile;

interface

uses
  { Net component created especially for the project }
  BackPropNet,
  Buttons,
  Classes,
  ComCtrls,
  Controls,
  Dialogs,
  ExtCtrls,
  FileDlg,
  Forms,
  Graphics,
  Messages,
  MPlayer,
  { Unit for storing values in dynamic arrays }
  SpeechRecogDynArrayFile,
  { Unit defining class with properties and methods for painting diagrams }
  SpeechRecogPaintDiagsFile,
  { Unit defining WaveFile properties and methods in class }
  SpeechRecogWaveFile,
  Spin,
  StdCtrls,
  SysUtils,
  Windows;

type
  { Class defining used form 'SpeechRecogForm' }
  TSpeechRecogForm = class(TForm)
    { TabSheet 'File' }
    FileBitBtnOpen: TBitBtn;
    FileEditFileWav: TEdit;
    FileComboBoxSegmCurve: TComboBox;
    FileComboBoxSegmWind: TComboBox;
    FileGroupBoxBark: TGroupBox;
    FileGroupBoxCent: TGroupBox;
    FileGroupBoxFileWav: TGroupBox;
    FileGroupBoxLoud: TGroupBox;
    FileGroupBoxNorm: TGroupBox;
    FileGroupBoxPow: TGroupBox;
    FileGroupBoxSegm: TGroupBox;
    FileGroupBoxWave: TGroupBox;
    FileGroupBoxWordDetect: TGroupBox;
    FileLabelBarkBandNo: TLabel;
    FileLabelBarkBandWidth: TLabel;
    FileLabelCentAverage: TLabel;
    FileLabelCentSum: TLabel;
    FileLabelFileWav: TLabel;
    FileLabelLoudExp: TLabel;
    FileLabelNormAbsMax: TLabel;
    FileLabelNormFactor: TLabel;
    FileLabelPowFrameNo: TLabel;
    FileLabelPowFrameWidth: TLabel;
    FileLabelPowSampleNo: TLabel;
    FileLabelPowWind: TLabel;
    FileLabelSegmChosenFrame: TLabel;
    FileLabelWaveAvgBytesPerSec: TLabel;
    FileLabelWaveAvgBytesPerSecUnit: TLabel;
    FileLabelWaveBitsPerSample: TLabel;
    FileLabelWaveBitsPerSampleUnit: TLabel;
    FileLabelWaveChannel: TLabel;
    FileLabelWaveDataChunkLength: TLabel;
    FileLabelWaveDataChunkLengthUnit: TLabel;
    FileLabelWaveFormat: TLabel;
    FileLabelWaveSamplesPerSec: TLabel;
    FileLabelWaveSamplesPerSecUnit: TLabel;
    FileLabelWordDetectPause: TLabel;
    FileLabelWordDetectSpeechLevel: TLabel;
    FileLabelWordDetectWord: TLabel;
    FileLabelXAxis: TLabel;
    FileLabelYAxis: TLabel;
    FileMediaPlayer: TMediaPlayer;
    FilePaintBox: TPaintBox;
    FilePanel: TPanel;
    FilePanelBarkBandNo: TPanel;
    FilePanelBarkBandWidth: TPanel;
    FilePanelCentAverage: TPanel;
    FilePanelCentSum: TPanel;
    FilePanelLoudExp: TPanel;
    FilePanelNormAbsMax: TPanel;
    FilePanelNormFactor: TPanel;
    FilePanelPowFrameNo: TPanel;
    FilePanelPowFrameWidth: TPanel;
    FilePanelPowSamples: TPanel;
    FilePanelPowWind: TPanel;
    FilePanelWaveAvgBytesPerSec: TPanel;
    FilePanelWaveBitsPerSample: TPanel;
    FilePanelWaveChannels: TPanel;
    FilePanelWaveDataChunkLength: TPanel;
    FilePanelWaveFormat: TPanel;
    FilePanelWaveSamplesPerSec: TPanel;
    FilePanelXAxis: TPanel;
    FilePanelYAxis: TPanel;
    FileProgressBar: TProgressBar;
    FileShapeMediaPlayer: TShape;
    FileSpinEditSegmChosenFrame: TSpinEdit;
    FileSpinEditWordDetectPause: TSpinEdit;
    FileSpinEditWordDetectSpeechLevel: TSpinEdit;
    FileSpinEditWordDetectWord: TSpinEdit;
    FileTabControl: TTabControl;
    FileTabSheet: TTabSheet;
    { TabSheet 'Pattern' }
    PattBevelHorizTable: TBevel;
    PattBevelPatternSet: TBevel;
    PattBevelVertTable: TBevel;
    PattBitBtnBreak: TBitBtn;
    PattBitBtnDelPatt: TBitBtn;
    PattBitBtnLearn: TBitBtn;
    PattBitBtnNewPatt: TBitBtn;
    PattBitBtnNewPatternSet: TBitBtn;
    PattBitBtnOpenPatternSet: TBitBtn;
    PattBitBtnSavePatternSet: TBitBtn;
    PattCheckBoxSave: TCheckBox;
    PattComboBoxPattCurr: TComboBox;
    PattEditFilePtn: TEdit;
    PattEditValTargCurr: TEdit;
    PattGroupBoxFilePtn: TGroupBox;
    PattGroupBoxNeuroNet: TGroupBox;
    PattGroupBoxPatt: TGroupBox;
    PattGroupBoxPattSet: TGroupBox;
    PattLabelFilePtn: TLabel;
    PattLabelInpSample: TLabel;
    PattLabelInpSampleWav: TLabel;
    PattLabelListInp: TLabel;
    PattLabelListTarg: TLabel;
    PattLabelNrPatt: TLabel;
    PattLabelPattCurr: TLabel;
    PattLabelPatternSetDim: TLabel;
    PattLabelPatternSetDimInp: TLabel;
    PattLabelPatternSetDimTarg: TLabel;
    PattLabelTargSample: TLabel;
    PattListBoxInp: TListBox;
    PattListBoxTarg: TListBox;
    PattPanelNrPatt: TPanel;
    PattPanelPatternSetDimInp: TPanel;
    PattPanelPatternSetDimTarg: TPanel;
    PattPanelSave: TPanel;
    PattPanelValInpCurr: TPanel;
    PattSpeedButtonNeuroNet: TSpeedButton;
    PattTabSheet: TTabSheet;
    { TabSheet 'Recall' }
    RecallBitBtnCfg: TBitBtn;
    RecallBitBtnLrn: TBitBtn;
    RecallBitBtnOffline: TBitBtn;
    RecallBitBtnOnline: TBitBtn;
    RecallBitBtnReset: TBitBtn;
    RecallBitBtnWav: TBitBtn;
    RecallGroupBoxNeuroNet: TGroupBox;
    RecallImageFlower: TImage;
    RecallImagePrince: TImage;
    RecallPanelAction: TPanel;
    RecallPanelActionFrame: TPanel;
    RecallPanelCfg: TPanel;
    RecallPanelLrn: TPanel;
    RecallPanelResult: TPanel;
    RecallPanelWav: TPanel;
    RecallRadioGroupType: TRadioGroup;
    RecallShape1: TShape;
    RecallShape2: TShape;
    RecallShape3: TShape;
    RecallShape4: TShape;
    RecallShape5: TShape;
    RecallShape6: TShape;
    RecallShape7: TShape;
    RecallShape8: TShape;
    RecallTabSheet: TTabSheet;
    { Section 'SpeechRecognition' }
    SpeechRecogBackPropNet: TBackPropNet;
    SpeechRecogFlowerImageList: TImageList;
    SpeechRecogLabelAuthor: TLabel;
    SpeechRecogNetFileDialog: TNetFileDialog;
    SpeechRecogPageControl: TPageControl;
    SpeechRecogPrinceImageList: TImageList;
    SpeechRecogStatusBar: TStatusBar;
    SpeechRecogTimer: TTimer;
    { TabSheet 'File' }
    procedure FileBitBtnOpenClick(Sender: TObject);
    procedure FileComboBoxSegmCurveExit(Sender: TObject);
    procedure FileComboBoxSegmCurveKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FileComboBoxSegmWindExit(Sender: TObject);
    procedure FileComboBoxSegmWindKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FileMediaPlayerClick(Sender: TObject; Button: TMPBtnType;
      var DoDefault: Boolean);
    procedure FilePaintBoxPaint(Sender: TObject);
    procedure FileSpinEditSegmChosenFrameExit(Sender: TObject);
    procedure FileSpinEditSegmChosenFrameKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FileSpinEditWordDetectPauseExit(Sender: TObject);
    procedure FileSpinEditWordDetectPauseKeyDown(Sender: TObject;
      var Key: Word; Shift: TShiftState);
    procedure FileSpinEditWordDetectSpeechLevelExit(Sender: TObject);
    procedure FileSpinEditWordDetectSpeechLevelKeyDown(Sender: TObject;
      var Key: Word; Shift: TShiftState);
    procedure FileSpinEditWordDetectWordExit(Sender: TObject);
    procedure FileSpinEditWordDetectWordKeyDown(Sender: TObject;
      var Key: Word; Shift: TShiftState);
    procedure FileTabControlChange(Sender: TObject);
    { Section 'Form' }
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    { TabSheet 'Pattern' }
    procedure PattBitBtnBreakClick(Sender: TObject);
    procedure PattBitBtnDelPattClick(Sender: TObject);
    procedure PattBitBtnLearnClick(Sender: TObject);
    procedure PattBitBtnNewPattClick(Sender: TObject);
    procedure PattBitBtnNewPatternSetClick(Sender: TObject);
    procedure PattBitBtnOpenPatternSetClick(Sender: TObject);
    procedure PattBitBtnSavePatternSetClick(Sender: TObject);
    procedure PattComboBoxPattCurrChange(Sender: TObject);
    procedure PattEditValTargCurrExit(Sender: TObject);
    procedure PattEditValTargCurrKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PattListBoxTargExit(Sender: TObject);
    procedure PattListBoxTargKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PattSpeedButtonNeuroNetClick(Sender: TObject);
    { TabSheet 'Recall' }
    procedure RecallBitBtnCfgClick(Sender: TObject);
    procedure RecallBitBtnLrnClick(Sender: TObject);
    procedure RecallBitBtnOfflineClick(Sender: TObject);
    procedure RecallBitBtnOnlineClick(Sender: TObject);
    procedure RecallBitBtnResetClick(Sender: TObject);
    procedure RecallBitBtnWavClick(Sender: TObject);
    procedure RecallImagePrinceClick(Sender: TObject);
    procedure RecallPanelActionClick(Sender: TObject);
    procedure RecallRadioGroupTypeClick(Sender: TObject);
    procedure RecallShape1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    { Section 'SpeechRecognition' }
    procedure SpeechRecogBackPropNetLearnExit(Sender: TBackPropNet;
      const ExitCode: TExitCode);
    procedure SpeechRecogPageControlChange(Sender: TObject);
    procedure SpeechRecogStatusBarDrawPanel(StatusBar: TStatusBar;
      Panel: TStatusPanel; const Rect: TRect);
    procedure SpeechRecogTimerDefeatTimer(Sender: TObject);
    procedure SpeechRecogTimerRecordTimer(Sender: TObject);
    procedure SpeechRecogTimerSuccessTimer(Sender: TObject);
  private
    { TabSheet 'File' }
    FilePaintDiags: TPaintDiags; // Class setting variables for diagrams
    FileWaveFile: TWaveFile; // Class containing WaveFile with all properties
    { TabSheet 'Pattern' }
    PattStringList: TStringList; // List containing WaveFileNames
    { StatusBar variables }
    SpeechRecogStatusImageIndex: Integer; // Index for StatusImage
    { TabSheet 'File' }
    procedure FileCalcSignalNew; // Calculate all values new
    procedure FileDrawCoOrdSystem(const Quadrants: Integer); virtual; // Draw Co-Ordinate System
    procedure FileHideBoxes; // Hide all GroupBoxes belonging to diagrams
    procedure FileLoadWaveFile(const FileName: TFileName); // Load new WaveFile
    procedure FilePaintBackGround(const Color: TColor); virtual; // Paint BackGround of PaintBox
    procedure FileReadWaveFileParams; // Read parameters of WaveFile and write them on form
    { TabSheet 'Recall' }
    procedure RecallActionDefeat; // Image action if recognition failed
    procedure RecallActionStandard; // Set images to standard position
    procedure RecallActionSuccess; // Image action if recognition succeeded
    procedure RecallResetImages; // Reset images
    procedure RecallResultCheck; // Check result of recognition
    { StatusBar }
    procedure SpeechRecogSetStatus(ImageIndex: Integer; Text: string); // Set status in panel
    procedure SpeechRecogStatusBarShowHint(Sender: TObject); // Show hints in panel
  public
  end;

  { Types for procedure importing BackPropagation from DLL - explicitely }
  TCallBackPropagation = procedure;

  { Special Exception class for errors during loading DLL }
  EDLLLoadError = class(Exception);

const
  crSpeechRecogWaitCursor = 2; // Cursor used during ongoing actions

{ Declare function to export SpeechRecogForm into DLL 'SpeechRecognition' }
procedure CallSpeechRecognition; export;

implementation

{$R *.DFM}
{$R SPEECHRECOG.RES}

{ Declare function to export SpeechRecogForm into DLL 'SpeechRecognition' }

procedure CallSpeechRecognition;
var
  SpeechRecogForm: TSpeechRecogForm;
begin
  try
    SpeechRecogForm := TSpeechRecogForm.Create(nil);
    try
      SpeechRecogForm.ShowModal;
    finally
      SpeechRecogForm.Free;
    end;
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg(E.ClassName + '(Fehler beim Erstellen des Formulars!)', mtError, [mbOk], 0);
    end;
  end;
end;

{ TabSheet 'File' }

procedure TSpeechRecogForm.FileBitBtnOpenClick(Sender: TObject);
begin
  { OpenDialog settings }
  with SpeechRecogNetFileDialog do try
    DialogMode := dmOpen;
    TemplateName := '';
    Title := 'ffnen einer AudioDatei';
    Filter := 'AudioDateien (*.wav)|*.wav';
    FileName := FileEditFileWav.Text;
    DefaultExt := 'wav';
    if (Execute = True) then begin
      FileLoadWaveFile(FileName);
      { Paint diagram and show GroupBoxes }
      if (SpeechRecogPageControl.ActivePage = FileTabSheet) then FileTabControlChange(Self);
      with FileWaveFile do begin
        FileEditFileWav.Text := ExtractFileName(WaveFileName);
        RecallPanelWav.Caption := ExtractFileName(WaveFileName);
      end;
    end;
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message , mtError, [mbOK], 0)
    end;
  end;
end;

procedure TSpeechRecogForm.FileCalcSignalNew;
begin
  { Step }
  FileProgressBar.StepBy(50);
  { Set status }
  SpeechRecogSetStatus(1, 'Berechne und zeichne neu');
  SpeechRecogStatusBar.Refresh;
  { Set WaitCursor }
  Screen.Cursor := crSpeechRecogWaitCursor;
  { Calculate signal new }
  with FileWaveFile do SignalProcessing;
  { Paint new }
  FilePaintBoxPaint(Self);
  { Step }
  FileProgressBar.StepBy(50);
  { Set status }
  SpeechRecogSetStatus(0, 'Warte auf Eingabe');
  SpeechRecogStatusBar.Refresh;
  { Reset WaitCursor }
  Screen.Cursor := crDefault;
end;

procedure TSpeechRecogForm.FileComboBoxSegmCurveExit(Sender: TObject);
begin
  if (FileWaveFile <> nil) then with FileWaveFile do begin
    with FileSpinEditSegmChosenFrame do begin
      with FileComboBoxSegmCurve do
        if ((ItemIndex = 3) or (ItemIndex = 4)) then begin
          MinValue := WordDetParams.WordBeginFrame;
          MaxValue := WordDetParams.WordEndFrame;
        end
        else begin
          MinValue := 1;
          MaxValue := FrameNo;
        end;
      if ((Value < MinValue) or (Value > MaxValue)) then begin
        Value := MinValue;
        FileSpinEditSegmChosenFrameExit(Self);
      end;
    end;
    if ((Sender is TComboBox) = True) then FileCalcSignalNew;
  end;
end;

procedure TSpeechRecogForm.FileComboBoxSegmCurveKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then FileComboBoxSegmCurveExit(Self);
end;

procedure TSpeechRecogForm.FileComboBoxSegmWindExit(Sender: TObject);
begin
  if (FileWaveFile <> nil) then with FileWaveFile do begin
    with FileComboBoxSegmWind do begin
      if (ItemIndex = 0) then WindowType := [wtBartlett];
      if (ItemIndex = 1) then WindowType := [wtBlackman];
      if (ItemIndex = 2) then WindowType := [wtHamming];
      if (ItemIndex = 3) then WindowType := [wtHann];
      if (ItemIndex = 4) then WindowType := [wtRectangle];
    end;
    if ((Sender is TComboBox) = True) then FileCalcSignalNew;
  end;
end;

procedure TSpeechRecogForm.FileComboBoxSegmWindKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then FileComboBoxSegmWindExit(Self);
end;

procedure TSpeechRecogForm.FileDrawCoOrdSystem(const Quadrants: Integer);
begin
  with FilePaintBox.Canvas do begin
    Pen.Width := 1;
    Pen.Style := psSolid;
    Pen.Color := clSilver;
    with FilePaintDiags do begin
      if (Quadrants = 1) then begin
        { X-axis and arrow }
        MoveTo(Left, Bottom); LineTo(Right, Bottom);
        MoveTo(Right, Bottom); LineTo((Right - 10), (Bottom - 3));
        MoveTo(Right, Bottom); LineTo((Right - 10), (Bottom + 3));
        { Y-axis and arrow }
        MoveTo(Left, Bottom); LineTo(Left, Top);
        MoveTo(Left, Top); LineTo((Left - 3), (Top + 10));
        MoveTo(Left, Top); LineTo((Left + 3), (Top + 10));
      end;
      if (Quadrants = 2) then begin
        { X-axis and arrow }
        MoveTo(Left, MidHoriz); LineTo(Right, MidHoriz);
        MoveTo(Right, MidHoriz); LineTo((Right - 10), (MidHoriz - 3));
        MoveTo(Right, MidHoriz); LineTo((Right - 10), (MidHoriz + 3));
        { Y-axis and arrow }
        MoveTo(Left, Bottom); LineTo(Left, Top);
        MoveTo(Left, Top); LineTo((Left - 3), (Top + 10));
        MoveTo(Left, Top); LineTo((Left + 3), (Top + 10));
      end;
    end;
  end;
end;

procedure TSpeechRecogForm.FileHideBoxes;
begin
  FileGroupBoxWave.Hide;
  FileGroupBoxCent.Hide;
  FileGroupBoxNorm.Hide;
  FileGroupBoxPow.Hide;
  FileGroupBoxWordDetect.Hide;
  FileGroupBoxSegm.Hide;
  FileGroupBoxBark.Hide;
  FileGroupBoxLoud.Hide;
end;

procedure TSpeechRecogForm.FileLoadWaveFile(const FileName: TFileName);
begin
  FileProgressBar.StepIt; // Step 1
  { Set WaitCursor }
  Screen.Cursor := crSpeechRecogWaitCursor;
  { Set status }
  SpeechRecogSetStatus(1, 'Lade Audio-Datei');
  SpeechRecogStatusBar.Refresh;
  { Free existing instance of TWaveFile }
  FileWaveFile.Free; FileWaveFile := nil;
  { Create new instance }
  FileWaveFile := TWaveFile.Create;
  FileProgressBar.StepIt; // Step 2
  try
    { Set status }
    SpeechRecogSetStatus(2, 'Bearbeite Signal');
    SpeechRecogStatusBar.Refresh;
    with FileWaveFile do begin
      { Set parameters for FileWaveFile  }
      FileComboBoxSegmWindExit(Self);
      FileComboBoxSegmCurveExit(Self);
      FileSpinEditSegmChosenFrameExit(Self);
      FileSpinEditWordDetectPauseExit(Self);
      FileSpinEditWordDetectSpeechLevelExit(Self);
      FileSpinEditWordDetectWordExit(Self);
      { Read from file and calculate data }
      LoadFromFile(FileName);
      FileProgressBar.StepBy(40); // Step 3-6
    end;
    { Write all values on form }
    FileReadWaveFileParams;
    FileMediaPlayer.EnabledButtons := [btPlay,btRecord];
    FileProgressBar.StepBy(40); // Step 7-10
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg(E.Message, mtError, [mbOk], 0);
    end;
  end;
  { Set status }
  SpeechRecogSetStatus(0, 'Warte auf Eingabe');
  SpeechRecogStatusBar.Refresh;
  { Reset WaitCursor }
  Screen.Cursor := crDefault;
end;

procedure TSpeechRecogForm.FileMediaPlayerClick(Sender: TObject;
  Button: TMPBtnType; var DoDefault: Boolean);
begin
  { Disable automatic start of operations by MediaPlayer }
  DoDefault := False;
  with FileMediaPlayer do begin
    case Button of
      btPlay: begin
        { Set status }
        SpeechRecogSetStatus(2, 'Wiedergabe luft');
        SpeechRecogStatusBar.Refresh;
        try
          FileName := FileWaveFile.WaveFileName;
          Open;
          EnabledButtons := [btStop];
          Wait := True;
          if (mpCanPlay in Capabilities) then Play;
          Stop;
          EnabledButtons := [btPlay, btRecord];
          Close;
        except
          on E: Exception do begin
            MessageBeep(0);
            MessageDlg(E.Message, mtError, [mbOk], 0);
          end;
        end;
        { Set status }
        SpeechRecogSetStatus(0, 'Warte auf Eingabe');
        SpeechRecogStatusBar.Refresh;
      end;
      btStop: begin
        case Mode of
          mpPlaying: begin
            try
              Stop;
              EnabledButtons := [btPlay, btRecord];
              Close;
            except
              on E: Exception do begin
                MessageBeep(0);
                MessageDlg(E.Message, mtError, [mbOk], 0);
              end;
            end;
            { Set status }
            SpeechRecogSetStatus(0, 'Warte auf Eingabe');
            SpeechRecogStatusBar.Refresh;
          end;
          mpRecording: begin
            SpeechRecogTimer.Enabled := False;
            try
              Stop;
              { Save complete wave file with header and wave data }
              Save;
              EnabledButtons := [btPlay, btRecord];
              Close;
            except
              on E: Exception do begin
                MessageBeep(0);
                MessageDlg(E.Message, mtError, [mbOk], 0);
              end;
            end;
            { Load new AudioFile }
            FileLoadWaveFile(FileName);
            { Paint diagram and show GroupBoxes }
            if (FileWaveFile <> nil) then
              if (SpeechRecogPageControl.ActivePage = FileTabSheet) then FileTabControlChange(Self);
              with FileWaveFile do begin
                FileEditFileWav.Text := ExtractFileName(WaveFileName);
                RecallPanelWav.Caption := ExtractFileName(WaveFileName);
              end;
            { Call procedure starting recall (after having hit BitBtnOnline) }
            if ((RecallBitBtnOnline.Enabled = True) and
              (SpeechRecogPageControl.ActivePage = RecallTabSheet)) then RecallResultCheck;
          end;
        end;
      end;
      btRecord: begin
        { Set status }
        SpeechRecogSetStatus(1, 'Aufnahme vorbereiten');
        SpeechRecogStatusBar.Refresh;
        { SaveDialog (don't use "with" as FileName is for MediaPlayer) }
        SpeechRecogNetFileDialog.DialogMode := dmSave;
        SpeechRecogNetFileDialog.TemplateName := '';
        SpeechRecogNetFileDialog.Title := 'Speichern einer AudioDatei';
        SpeechRecogNetFileDialog.Filter := 'AudioDateien (*.wav)|*.wav';
        SpeechRecogNetFileDialog.FileName := FileEditFileWav.Text;
        SpeechRecogNetFileDialog.DefaultExt := 'wav';
        if (SpeechRecogNetFileDialog.Execute = True) then begin
          FileWaveFile.Free; FileWaveFile := nil;
          FileWaveFile := TWaveFile.Create;
          FileWaveFile.WaveFileName := SpeechRecogNetFileDialog.FileName;
          { Create file by writing its header }
          FileWaveFile.SaveToFile(FileWaveFile.WaveFileName);
        end
        else if (FileWaveFile = nil) then begin
          MessageBeep(0);
          raise Exception.Create('Keine Audio-Datei angegeben!');
        end;
        try
          { Set FileName of MediaPlayer }
          FileName := FileWaveFile.WaveFileName;
          Open;
          MessageBeep(0);
          if (MessageDlg('Aufnahme starten?', mtConfirmation,
          [mbYes, mbNo], 0) = mrYes) then begin
            { Set status }
            SpeechRecogSetStatus(2, 'Aufnahme luft');
            SpeechRecogStatusBar.Refresh;
            EnabledButtons := [btStop];
            { Set Timer to automatically stop recording }
            with SpeechRecogTimer do begin
              OnTimer := SpeechRecogTimerRecordTimer;
              Interval := 3000;
              Enabled := True;
            end;
            StartRecording;
          end
          else begin
            try
              Close;
            except
              on E: Exception do begin
                MessageBeep(0);
                MessageDlg(E.Message, mtError, [mbOk], 0);
              end;
            end;
            { Set status }
            SpeechRecogSetStatus(0, 'Warte auf Eingabe');
            SpeechRecogStatusBar.Refresh;
          end;
        except
          on E: Exception do begin
            MessageBeep(0);
            MessageDlg(E.Message, mtError, [mbOk], 0);
          end;
        end;
      end;
    end;
  end;
end;

procedure TSpeechRecogForm.FilePaintBackGround(const Color: TColor);
begin
  with FilePaintBox.Canvas do begin
    Brush.Color := Color;
    FillRect(Rect(0, 0, Width, Height));
  end;
end;

procedure TSpeechRecogForm.FilePaintBoxPaint(Sender: TObject);

  { Logarithmise Value to Base }
  function Logarithmise(const Base: Double; const Value: Double): Double;
  begin
    try
      Result := (ln(Value) / ln(Base));
    except
      on Exception do Result := 0;
    end;
  end;

var
  i: Integer;
  HelpString: string;
begin
  with FilePaintBox.Canvas do begin
    Invalidate;
    if (FileWaveFile <> nil) then with FileWaveFile do begin
      if (FilePaintDiags <> nil) then with FilePaintDiags do begin
        FilePaintBackGround($00420000);
        { DataWave = wave signal }
        if (FileTabControl.TabIndex = 0) then begin
          DiagType := 0;
          SetDiagData(TDoubleArray(DataWave));
          FileDrawCoOrdSystem(2);
          { Display values }
          TextOut((Right - 120), (Bottom + 5), 'Abtastungen: ' + IntToStr(Round(MaxX)));
          if (MinY > 10) then Str(MinY : 5 : 2, HelpString)
          else Str(MinY : 1 : 4, HelpString);
          TextOut((Left - 5), (Bottom + 5), 'Minimum: ' + HelpString);
          if (MaxY > 10) then Str(MaxY : 5 : 2, HelpString)
          else Str(MaxY : 1 : 4, HelpString);
          TextOut((Left - 5), (Top - 25), 'Maximum: ' + HelpString);
          { Draw data }
          if ((DataWave <> nil) and (DataWave.Count > 1)) then begin
            Pen.Style := psSolid;
            Pen.Color := $00C6F7FF;
            MoveTo(Left, (MidHoriz - Trunc(DataWave[0] * FactY)));
            for i := 1 to (DataWave.Count - 1) do
              LineTo((Left + Trunc(i * FactX)), (MidHoriz - Trunc(DataWave[i] * FactY)));
          end;
        end;
        { DataCent = centralized signal }
        if (FileTabControl.TabIndex = 1) then begin
          DiagType := 1;
          SetDiagData(DataCent);
          FileDrawCoOrdSystem(2);
          { Display values }
          TextOut((Right - 120), (Bottom + 5), 'Abtastungen: ' + IntToStr(Round(MaxX)));
          if (MinY > 10) then Str(MinY : 5 : 2, HelpString)
          else Str(MinY : 1 : 4, HelpString);
          TextOut((Left - 5), (Bottom + 5), 'Minimum: ' + HelpString);
          if (MaxY > 10) then Str(MaxY : 5 : 2, HelpString)
          else Str(MaxY : 1 : 4, HelpString);
          TextOut((Left - 5), (Top - 25), 'Maximum: ' + HelpString);
          { Draw data }
          if ((DataCent <> nil) and (DataCent.Count > 1)) then begin
            Pen.Style := psSolid;
            Pen.Color := $00C6F7FF;
            MoveTo(Left, (MidHoriz - Trunc(DataCent[0] * FactY)));
            for i := 1 to (DataCent.Count - 1) do
              LineTo((Left + Trunc(i * FactX)), (MidHoriz - Trunc(DataCent[i] * FactY)));
          end;
        end;
        { DataNorm = normalized signal }
        if (FileTabControl.TabIndex = 2) then begin
          DiagType := 2;
          SetDiagData(DataNorm);
          FileDrawCoOrdSystem(2);
          { Display values }
          TextOut((Right - 120), (Bottom + 5), 'Abtastungen: ' + IntToStr(Round(MaxX)));
          if (MinY > 10) then Str(MinY : 5 : 2, HelpString)
          else Str(MinY : 1 : 4, HelpString);
          TextOut((Left - 5), (Bottom + 5), 'Minimum: ' + HelpString);
          if (MaxY > 10) then Str(MaxY : 5 : 2, HelpString)
          else Str(MaxY : 1 : 4, HelpString);
          TextOut((Left - 5), (Top - 25), 'Maximum: ' + HelpString);
          { Draw data }
          if ((DataNorm <> nil) and (DataNorm.Count > 1)) then begin
            Pen.Style := psSolid;
            Pen.Color := $00C6F7FF;
            MoveTo(Left, (MidHoriz - Trunc(DataNorm[0] * FactY)));
            for i := 1 to (DataNorm.Count - 1) do
              LineTo((Left + Trunc(i * FactX)), (MidHoriz - Trunc(DataNorm[i] * FactY)));
          end;
        end;
        { DataPow = power spectrum }
        if (FileTabControl.TabIndex = 3) then begin
          DiagType := 3;
          SetDiagData(DataPow);
          FileDrawCoOrdSystem(1);
          { Display values }
          TextOut((Right - 120), (Bottom + 5), 'Abtastungen: ' + IntToStr(Round(MaxX)));
          if (MinY > 10) then Str(MinY : 5 : 2, HelpString)
          else Str(MinY : 1 : 4, HelpString);
          TextOut((Left - 5), (Bottom + 5), 'Minimum: ' + HelpString);
          if (MaxY > 10) then Str(MaxY : 5 : 2, HelpString)
          else Str(MaxY : 1 : 4, HelpString);
          TextOut((Left - 5), (Top - 25), 'Maximum: ' + HelpString);
          { Draw data }
          if ((DataPow <> nil) and (DataPow.Count > 1)) then begin
            Pen.Color := $00C6F7FF;
            Pen.Style := psSolid;
            { Check if < 1 to avoid negative log-values }
            if (DataPow[0] < 1) then MoveTo(Left, Bottom)
            else MoveTo(Left, (Bottom - Trunc((20 * Logarithmise(10, DataPow[0])) * FactY)));
            for i := 1 to (DataPow.Count - 1) do
              if (DataPow[i] < 1) then LineTo((Left + Trunc(i * FactX)), Bottom)
              else LineTo((Left + Trunc(i * FactX)), (Bottom - Trunc((20 * Logarithmise(10, DataPow[i])) * FactY)));
            Pen.Color := clSilver;
            Pen.Style := psDot;
            with WordDetParams do begin
              MoveTo(Left, (Bottom - Round((20 * Logarithmise(10, SpeechLevel)) * FactY)));
              LineTo(Right, (Bottom - Round((20 * Logarithmise(10, SpeechLevel)) * FactY)));
              for i := 0 to (DataPow.Count - 1) do begin
                if (i = WordBeginFrame) then begin
                  MoveTo((Left + Trunc(i * FactX)), Bottom);
                  LineTo((Left + Trunc(i * FactX)), Top);
                  TextOut((Left + Trunc(i * FactX)), (Bottom + 5), 'WortAnfang: ' + IntToStr(WordBeginFrame));
                end;
                if (i = WordEndFrame) then begin
                  MoveTo((Left + Trunc(i * FactX)), Bottom);
                  LineTo((Left + Trunc(i * FactX)), Top);
                  TextOut((Left + Trunc(i * FactX)), (Bottom + 5), 'WortEnde: ' + IntToStr(WordEndFrame));
                end;
              end;
            end;
          end;
        end;
        { DataCut/DataWind/DataPow/DataBark/DataLoud for 1 Frame only }
        if (FileTabControl.TabIndex = 4) then begin
          { DataCutFrame }
          if (FileComboBoxSegmCurve.ItemIndex = 0) then begin
            DiagType := 4;
            SetDiagData(DataCutFrame);
            FileDrawCoOrdSystem(2);
            { Display values }
            TextOut((Right - 120), (Bottom + 5), 'Abtastungen: ' + IntToStr(Round(MaxX)));
            if (MinY > 10) then Str(MinY : 5 : 2, HelpString)
            else Str(MinY : 1 : 4, HelpString);
            TextOut((Left - 5), (Bottom + 5), 'Minimum: ' + HelpString);
            if (MaxY > 10) then Str(MaxY : 5 : 2, HelpString)
            else Str(MaxY : 1 : 4, HelpString);
            TextOut((Left - 5), (Top - 25), 'Maximum: ' + HelpString);
            { Draw data }
            if ((DataCutFrame <> nil) and (DataCutFrame.Count > 1)) then begin
              Pen.Style := psSolid;
              Pen.Color := $00C6F7FF;
              MoveTo(Left, (MidHoriz - Trunc(DataCutFrame[0] * FactY)));
              for i := 1 to (DataCutFrame.Count - 1) do
                LineTo((Left + Trunc(i * FactX)), (MidHoriz - Trunc(DataCutFrame[i] * FactY)));
            end;
          end;
          { DataWindFrame }
          if (FileComboBoxSegmCurve.ItemIndex = 1) then begin
            DiagType := 4;
            SetDiagData(DataWindFrame);
            FileDrawCoOrdSystem(2);
            { Display values }
            TextOut((Right - 120), (Bottom + 5), 'Fensterbreite: ' + IntToStr(Round(MaxX)));
            if (MinY > 10) then Str(MinY : 5 : 2, HelpString)
            else Str(MinY : 1 : 4, HelpString);
            TextOut((Left - 5), (Bottom + 5), 'Minimum: ' + HelpString);
            if (MaxY > 10) then Str(MaxY : 5 : 2, HelpString)
            else Str(MaxY : 1 : 4, HelpString);
            TextOut((Left - 5), (Top - 25), 'Maximum: ' + HelpString);
            { Draw data }
            if ((DataWindFrame <> nil) and (DataWindFrame.Count > 1)) then begin
              Pen.Style := psSolid;
              Pen.Color := $00C6F7FF;
              MoveTo(Left, (MidHoriz - Trunc(DataWindFrame[0] * FactY)));
              for i := 1 to (DataWindFrame.Count - 1) do
                LineTo((Left + Trunc(i * FactX)), (MidHoriz - Trunc(DataWindFrame[i] * FactY)));
            end;
          end;
          { DataPowFrame }
          if (FileComboBoxSegmCurve.ItemIndex = 2) then begin
            DiagType := 5;
            SetDiagData(DataPowFrame);
            FileDrawCoOrdSystem(1);
            { Display values }
            TextOut((Left + 10), (Bottom + 5), 'Relevante Information');
            TextOut((Left + 360), (Bottom + 5), 'Redundante Information');
            TextOut((Right - 120), (Bottom + 5), 'Sttzstellen: ' + IntToStr(Round(MaxX)));
            if (MaxY > 10) then Str(MaxY : 5 : 2, HelpString)
            else Str(MaxY : 1 : 4, HelpString);
            TextOut((Left - 5), (Top - 25), 'Maximum: ' + HelpString);
            { Draw data }
            Brush.Color := $00C6F7FF;
            if ((DataPowFrame <> nil) and (DataPowFrame.Count > 1)) then
              for i := 0 to (DataPowFrame.Count - 1) do
                FillRect(Rect((Left + ((i * 2) + 1)), Bottom,
                (Left + ((i * 2) + 2)), (Bottom - Trunc(DataPowFrame[i] * FactY))));
          end;
          { DataBarkFrame }
          if (FileComboBoxSegmCurve.ItemIndex = 3) then begin
            DiagType := 5;
            SetDiagData(DataBarkFrame);
            FileDrawCoOrdSystem(1);
            if ((DataBarkFrame <> nil) and (DataBarkFrame.Count > 1)) then begin
              { Display values }
              for i := 0 to (DataBarkFrame.Count - 1) do begin
                TextOut((Left + ((i * 40) + 5)), (Bottom + 5), IntToStr(i));
                if (DataBarkFrame[i] >= 100)
                then HelpString := IntToStr(Trunc(DataBarkFrame[i]))
                else Str(DataBarkFrame[i] : 4 : 2, HelpString);
                TextOut((Left + ((i * 40) + 5)), (Top - 25), HelpString);
              end;
              { Draw data }
              Brush.Color := $00C6F7FF;
              for i := 0 to (DataBarkFrame.Count - 1) do
                FillRect(Rect((Left + ((i * 40) + 5)), Bottom,
                (Left + ((i * 40) + 35)), (Bottom - Trunc(DataBarkFrame[i] * FactY))));
            end;
          end;
          { DataLoudFrame }
          if (FileComboBoxSegmCurve.ItemIndex = 4) then begin
            DiagType := 6;
            SetDiagData(DataLoudFrame);
            FileDrawCoOrdSystem(1);
            if ((DataLoudFrame <> nil) and (DataLoudFrame.Count > 1)) then begin
              { Display values }
              for i := 0 to (DataLoudFrame.Count - 1) do begin
                TextOut((Left + ((i * 40) + 5)), (Bottom + 5), IntToStr(i));
                if (DataLoudFrame[i] >= 100)
                then HelpString := IntToStr(Trunc(DataLoudFrame[i]))
                else Str(DataLoudFrame[i] : 4 : 2, HelpString);
                TextOut((Left + ((i * 40) + 5)), (Top - 25), HelpString);
              end;
              { Draw data }
              Brush.Color := $00C6F7FF;
              for i := 0 to (DataLoudFrame.Count - 1) do
                FillRect(Rect((Left + ((i * 40) + 5)), Bottom,
                (Left + ((i * 40) + 35)), (Bottom - Trunc(DataLoudFrame[i] * FactY))));
            end;
          end;
        end;
        { DataBark (= Bark spectrum) }
        if (FileTabControl.TabIndex = 5) then begin
          DiagType := 5;
          SetDiagData(DataBark);
          FileDrawCoOrdSystem(1);
          if ((DataBark <> nil) and (DataBark.Count > 1)) then begin
            { Display values }
            for i := 0 to (DataBark.Count - 1) do begin
              TextOut((Left + ((i * 40) + 5)), (Bottom + 5), IntToStr(i));
              if (DataBark[i] >= 100)
              then HelpString := IntToStr(Trunc(DataBark[i]))
              else Str(DataBark[i] : 4 : 2, HelpString);
              TextOut((Left + ((i * 40) + 5)), (Top - 25), HelpString);
            end;
            { Draw data }
            Brush.Color := $00C6F7FF;
            for i := 0 to (DataBark.Count - 1) do
              FillRect(Rect((Left + ((i * 40) + 5)), Bottom,
              (Left + ((i * 40) + 35)), (Bottom - Trunc(DataBark[i] * FactY))));
          end;
        end;
        { DataLoud (= loudness) }
        if (FileTabControl.TabIndex = 6) then begin
          DiagType := 6;
          SetDiagData(DataLoud);
          FileDrawCoOrdSystem(1);
          if ((DataLoud <> nil) and (DataLoud.Count > 1)) then begin
            { Display values }
            for i := 0 to (DataLoud.Count - 1) do begin
              TextOut((Left + ((i * 40) + 5)), (Bottom + 5), IntToStr(i));
              if (DataLoud[i] >= 100)
              then HelpString := IntToStr(Trunc(DataLoud[i]))
              else Str(DataLoud[i] : 4 : 2, HelpString);
              TextOut((Left + ((i * 40) + 5)), (Top - 25), HelpString);
            end;
            { Draw data }
            Brush.Color := $00C6F7FF;
            for i := 0 to (DataLoud.Count - 1) do
              FillRect(Rect((Left + ((i * 40) + 5)), Bottom,
              (Left + ((i * 40) + 35)), (Bottom - Trunc(DataLoud[i] * FactY))));
          end;
        end;
      end;
    end;
  end;
end;

procedure TSpeechRecogForm.FileReadWaveFileParams;
begin
  with FileWaveFile do begin
    { Write file parameters on form }
    with WaveFormat do begin
      FilePanelWaveChannels.Caption := IntToStr(nChannels);
      if (wFormatTag = 1) then FilePanelWaveFormat.Caption := 'PCM';
      FilePanelWaveSamplesPerSec.Caption := IntToStr(nSamplesPerSec);
      FilePanelWaveAvgBytesPerSec.Caption := IntToStr(nAvgBytesPerSec);
      FilePanelWaveBitsPerSample.Caption := IntToStr(nBitsPerSample);
      FilePanelWaveDataChunkLength.Caption := IntToStr(DataChunkLength);
    end;
    FilePanelCentSum.Caption := FloatToStr(Round(Sum));
    FilePanelCentAverage.Caption := FloatToStr(Round(Average));
    FilePanelNormAbsMax.Caption := FloatToStr(Round(AbsMax));
    FilePanelNormFactor.Caption := FloatToStr(Round(Factor));
    FilePanelPowSamples.Caption := IntToStr(SampleNo);
    FilePanelPowFrameNo.Caption := IntToStr(FrameNo);
    FilePanelPowFrameWidth.Caption := IntToStr(FrameWidth);
    with FileComboBoxSegmWind do begin
      if (WindowType = [wtBartlett]) then begin
        ItemIndex := 0;
        FilePanelPowWind.Caption := 'Bartlett';
      end;
      if (WindowType = [wtBlackman]) then begin
        ItemIndex := 1;
        FilePanelPowWind.Caption := 'Blackman';
      end;
      if (WindowType = [wtHamming]) then begin
        ItemIndex := 2;
        FilePanelPowWind.Caption := 'Hamming';
      end;
      if (WindowType = [wtHann]) then begin
        ItemIndex := 3;
        FilePanelPowWind.Caption := 'Hann';
      end;
      if (WindowType = [wtRectangle]) then begin
        ItemIndex := 4;
        FilePanelPowWind.Caption := 'Rechteck';
      end;
    end;
    FileComboBoxSegmCurveExit(Self);
    with WordDetParams do begin
      FileSpinEditWordDetectPause.Value := PauseCount;
      FileSpinEditWordDetectSpeechLevel.Value := SpeechLevel;
      FileSpinEditWordDetectWord.Value := WordCount;
    end;
    FilePanelBarkBandNo.Caption := '18';
    FilePanelBarkBandWidth.Caption := 'gehrrichtig; nichtlinear';
    FilePanelLoudExp.Caption := '0,5';
  end;
end;

procedure TSpeechRecogForm.FileSpinEditSegmChosenFrameExit(Sender: TObject);
begin
  if (FileWaveFile <> nil) then with FileWaveFile do
    ChosenFrame := FileSpinEditSegmChosenFrame.Value;
  if ((Sender is TSpinEdit) = True) then FileCalcSignalNew;
end;

procedure TSpeechRecogForm.FileSpinEditSegmChosenFrameKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then FileSpinEditSegmChosenFrameExit(Self);
end;

procedure TSpeechRecogForm.FileSpinEditWordDetectPauseExit(Sender: TObject);
begin
  if (FileWaveFile <> nil) then with FileWaveFile.WordDetParams do
    PauseCount := FileSpinEditWordDetectPause.Value;
  if ((Sender is TSpinEdit) = True) then FileCalcSignalNew;
end;

procedure TSpeechRecogForm.FileSpinEditWordDetectPauseKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then FileSpinEditWordDetectPauseExit(Self);
end;

procedure TSpeechRecogForm.FileSpinEditWordDetectSpeechLevelExit(Sender: TObject);
begin
  if (FileWaveFile <> nil) then with FileWaveFile.WordDetParams do
    SpeechLevel := FileSpinEditWordDetectSpeechLevel.Value;
  if ((Sender is TSpinEdit) = True) then FileCalcSignalNew;
end;

procedure TSpeechRecogForm.FileSpinEditWordDetectSpeechLevelKeyDown(
  Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then FileSpinEditWordDetectSpeechLevelExit(Self);
end;

procedure TSpeechRecogForm.FileSpinEditWordDetectWordExit(Sender: TObject);
begin
  if (FileWaveFile <> nil) then with FileWaveFile.WordDetParams do
    WordCount := FileSpinEditWordDetectWord.Value;
  if ((Sender is TSpinEdit) = True) then FileCalcSignalNew;
end;

procedure TSpeechRecogForm.FileSpinEditWordDetectWordKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then FileSpinEditWordDetectWordExit(Self);
end;

procedure TSpeechRecogForm.FileTabControlChange(Sender: TObject);
begin
  { Set GroupBoxes belonging to diagram }
  FileHideBoxes;
  case FileTabControl.TabIndex of
    0: begin
      FileGroupBoxWave.Show;
      FilePanelYAxis.Caption := 'Signal [Integer]';
      FilePanelXAxis.Caption := 't [s]';
    end;
    1: begin
      FileGroupBoxCent.Show;
      FilePanelYAxis.Caption := 'Signal zentriert';
      FilePanelXAxis.Caption := 't [s]';
    end;
    2: begin
      FileGroupBoxNorm.Show;
      FilePanelYAxis.Caption := 'Signal normiert';
      FilePanelXAxis.Caption := 't [s]';
    end;
    3: begin
      FileGroupBoxPow.Show;
      FileGroupBoxWordDetect.Show;
      FilePanelYAxis.Caption := 'Leistung P';
      FilePanelXAxis.Caption := 't [s]';
    end;
    4: begin
      FileGroupBoxSegm.Show;
      if (FileComboBoxSegmCurve.ItemIndex = 0) then begin
        FilePanelYAxis.Caption := 'Signal normiert';
        FilePanelXAxis.Caption := 't [s]';
      end;
      if (FileComboBoxSegmCurve.ItemIndex = 1) then begin
        FilePanelYAxis.Caption := 'Signal gefenstert';
        FilePanelXAxis.Caption := 't [s]';
      end;
      if (FileComboBoxSegmCurve.ItemIndex = 2) then begin
        FilePanelYAxis.Caption := 'Leistung P';
        FilePanelXAxis.Caption := 'f [Hz]';
      end;
      if (FileComboBoxSegmCurve.ItemIndex = 3) then begin
        FilePanelYAxis.Caption := 'Leistung P';
        FilePanelXAxis.Caption := 'Tonheit [Bark]';
      end;
      if (FileComboBoxSegmCurve.ItemIndex = 4) then begin
        FilePanelYAxis.Caption := 'Lautheit';
        FilePanelXAxis.Caption := 'Tonheit [Bark]';
      end;
    end;
    5: begin
      FileGroupBoxBark.Show;
      FilePanelYAxis.Caption := 'Leistung P';
      FilePanelXAxis.Caption := 'Tonheit [Bark]';
    end;
    6: begin
      FileGroupBoxLoud.Show;
      FilePanelYAxis.Caption := 'Lautheit';
      FilePanelXAxis.Caption := 'Tonheit [Bark]';
    end;
  end;
  { Paint chosen diagram }
  FilePaintBoxPaint(Self);
end;

{ Section 'Form' following }

procedure TSpeechRecogForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  { Set status }
  SpeechRecogSetStatus(1, 'Anwendung beenden');
  SpeechRecogStatusBar.Refresh;
  { Finishing dialog }
  MessageBeep(0);
  if (MessageDlg('Formular "Spracherkennung" schliessen? ' +
    'Alle Daten gehen verloren, insofern nicht vorher gespeichert!',
    mtConfirmation, [mbYes, mbNo], 0) = mrYes) then CanClose := True
  else begin
    CanClose := False;
    { Set status }
    SpeechRecogSetStatus(0, 'Warte auf Eingabe');
    SpeechRecogStatusBar.Refresh;
  end;
end;

procedure TSpeechRecogForm.FormCreate(Sender: TObject);
var
  ExeFilePath: string;
begin
  { Check for authoring identification - copyright }
  if (SpeechRecogLabelAuthor.Caption <> 'ch') then FormDestroy(Self);
  with Application do begin
    { Get path of Application }
    ExeFilePath := ExtractFilePath(ExeName);
    { Write hints in StatusBar }
    OnHint := SpeechRecogStatusBarShowHint;
    { Set HelpFile }
    HelpFile := ExeFilePath + 'Help\SpeRecog.hlp';
  end;
  { Load WaitCursor }
  Screen.Cursors[crSpeechRecogWaitCursor] := LoadCursor(HInstance, 'Wait');
  { Load images for ResultPresentation }
  with SpeechRecogPrinceImageList do try
    ResourceLoad(rtBitmap, ('PrinceStandard'), clOlive);
    ResourceLoad(rtBitmap, ('PrinceSuccess'), clOlive);
    ResourceLoad(rtBitmap, ('PrinceDefeat'), clOlive);
  except
    on EInvalidGraphic do begin
      MessageBeep(0);
      MessageDlg('Kein Graphik-Datei-Format!', mtWarning, [mbOk], 0);
    end;
  end;
  { Load images for ResultPresentation }
  with SpeechRecogFlowerImageList do try
    ResourceLoad(rtBitmap, ('FlowerStandard'), clOlive);
    ResourceLoad(rtBitmap, ('FlowerSuccess'), clOlive);
    ResourceLoad(rtBitmap, ('FlowerDefeat'), clOlive);
  except
    on EInvalidGraphic do begin
      MessageBeep(0);
      MessageDlg('Keine Graphik-Datei!', mtWarning, [mbOk], 0);
    end;
  end;
  { Set ActivePage }
  SpeechRecogPageControl.ActivePage := FileTabSheet;
  { Set component contents }
  FileSpinEditWordDetectSpeechLevel.Value := 100;
  FileSpinEditWordDetectWord.Value := 5;
  FileSpinEditWordDetectPause.Value := 20;
  FileComboBoxSegmWind.ItemIndex := 2; // HammingWindow
  FileComboBoxSegmCurve.ItemIndex := 0;
  FileSpinEditSegmChosenFrame.Value := 0;
  { New instance of TPaintDiags necessary to paint diagrams }
  FilePaintDiags := TPaintDiags.Create;
  with FilePaintDiags do begin
    Bottom := (FilePaintBox.Height - 30);
    Left := 10;
    Right := (FilePaintBox.Width - 10);
    Top := 30;
    MidHoriz := (((FilePaintBox.Height - 60) div 2) + Top);
    MidVert := (((FilePaintBox.Width - 20) div 2) + Left);
    DiagType := 0;
    FactX := 0;
    FactY := 0;
    MaxX := 0;
    MaxY := 0;
    MinX := 0;
    MinY := 0;
  end;
  { New instance of TWaveFile }
  FileWaveFile := TWaveFile.Create;
  { Default settings for page 'File' }
  FileEditFileWav.Text := 'Untitled.wav';
  FileTabControl.TabIndex := 0;
  FileTabControlChange(Self);
  { New instance of TStringList }
  PattStringList := TStringList.Create;
  PattBitBtnNewPatternSetClick(Self);
  { Write in Panels }
  RecallPanelCfg.Caption := 'Vorgabe (720-8-8)';
  RecallPanelLrn.Caption := 'Keine';
  RecallPanelWav.Caption := 'Keine';
  { Set ResultImages }
  RecallActionStandard;
  { Set status }
  SpeechRecogSetStatus(0, 'Warte auf Eingabe');
  SpeechRecogStatusBar.Refresh;
end;

procedure TSpeechRecogForm.FormDestroy(Sender: TObject);
begin
  FileWaveFile.Free; FileWaveFile := nil;
  FilePaintDiags.Free; FilePaintDiags := nil;
  PattStringList.Free; PattStringList := nil;
end;

{ TabSheet 'Pattern' following }

procedure TSpeechRecogForm.PattBitBtnBreakClick(Sender: TObject);
begin
  { Interrupt learning process }
  SpeechRecogBackPropNet.Break := True;
end;

procedure TSpeechRecogForm.PattBitBtnDelPattClick(Sender: TObject);
begin
  with SpeechRecogBackPropNet.PatternSet do begin
    { Delete last pattern of pattern set? }
    if (PatternNo < 2) then begin
      MessageBeep(0);
      if (MessageDlg('Nur noch 1 Muster im Muster-Set. Lschen?',
        mtConfirmation, [mbYes, mbNo], 0) = mrYes) then begin
        { Delete all existing patterns }
        DeleteAll;
        { Delete all WaveFileNames }
        PattStringList.Clear;
        { Delete number of patterns in ComboBox }
        PattComboBoxPattCurr.Clear;
      end;
    end
    else if (PatternNo > 1) then begin
      { Delete pattern in net component }
      Delete(PattComboBoxPattCurr.ItemIndex);
      with PattComboBoxPattCurr do begin
        { Delete WaveFileName }
        PattStringList.Delete(ItemIndex);
        { Change data in ComboBox }
        Items.Delete(Items.Count - 1);
        ItemIndex := (Items.Count - 1);
      end;
    end;
    { Setting panel showing total number of patterns }
    PattPanelNrPatt.Caption := IntToStr(PatternNo);
  end;
  { Set values in ListBoxes }
  PattComboBoxPattCurrChange(Self);
end;

procedure TSpeechRecogForm.PattBitBtnLearnClick(Sender: TObject);
begin
  try
    { Set WaitCursor }
    Screen.Cursor := crSpeechRecogWaitCursor;
    { Set status }
    SpeechRecogSetStatus(1, 'bergebe Parameter');
    SpeechRecogStatusBar.Refresh;
    with SpeechRecogBackPropNet do begin
      { Set procedures for events within net component }
      OnExit := SpeechRecogBackPropNetLearnExit;
      OnInterval := nil;
      { Set interval in general to 1 }
      Interval := 1;
      { Reset all learn parameters to random values }
      ResetAll;
      { Set status }
      SpeechRecogSetStatus(2, 'Lerne Netz an');
      SpeechRecogStatusBar.Refresh;
      { Start learning process of the net }
      Learn;
    end;
    { Save data in file }
    if (PattCheckBoxSave.Checked = True) then
      with SpeechRecogNetFileDialog do try
        TemplateName := 'NetDialog';
        DialogMode := dmSave;
        Title := 'Speichern einer LernDatenDatei';
        { Filter on Learn }
        Filter := DefaultFilter;
        FilterIndex := 3;
        FileName := 'Untitled';
        DefaultExt := 'lrn';
        if Execute then SpeechRecogBackPropNet.SaveToFile(FileName);
        RecallPanelLrn.Caption := ExtractFileName(FileName);
      except
        on E: Exception do begin
          MessageBeep(0);
          MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message , mtError, [mbOK], 0)
        end;
      end
    else RecallPanelLrn.Caption := '...im aktuellen Netz';
  finally
    { Set status }
    SpeechRecogSetStatus(0, 'Warte auf Eingabe');
    SpeechRecogStatusBar.Refresh;
    { Reset WaitCursor }
    Screen.Cursor := crDefault;
  end;
end;

procedure TSpeechRecogForm.PattBitBtnNewPattClick(Sender: TObject);

  { Involve Base to the Exponent }
  function Involve(const Base: Double; const Exponent: Double): Double;
  begin
    try
      Result := Exp(Ln(Base) * Exponent);
    except
      on Exception do Result := 0;
    end;
  end;

var
  j, k, n: Integer;
  Result: Integer;
begin
  try
    { Call procedure from page 'File' to load FileWaveFile }
    FileBitBtnOpenClick(Self);
    { Check if AudioFile is loaded }
    if (FileWaveFile = nil) then begin
      MessageBeep(0);
      raise Exception.Create('Keine Audio-Datei geladen!');
    end;
    { Set ComboBox Items/Index }
    with PattComboBoxPattCurr do begin
      Items.Add(IntToStr(Items.Count + 1));
      ItemIndex := (Items.Count - 1);
    end;
    with SpeechRecogBackPropNet.PatternSet do begin
      { Create new pattern at the end of the set }
      PatternNo := (PatternNo + 1);
      { Hand over signal processing results to PatternSet of BackPropNet }
      with FileWaveFile do begin
        for j := 0 to (DataLoud.Count - 1) do begin
          Result := 0;
          for k := 0 to (39 - 1) do
            if ((DataLoud[j] > (k * 50)) and (DataLoud[j] <= ((k + 1) * 50))) then Result := k;
          k := 39;
          if (DataLoud[j] > (k * 50)) then Result := k;
          for n := 0 to 39 do
            if (n <= Result) then
              Items[PattComboBoxPattCurr.ItemIndex].Inputs[(j * 40) + n] := 1;
        end;
      end;
      { Setting panels showing total number of patterns and FileName }
      PattPanelNrPatt.Caption := IntToStr(PatternNo);
    end;
    PattStringList.Add(ExtractFileName(FileWaveFile.WaveFileName));
    PattPanelValInpCurr.Caption := PattStringList.Strings[PattComboBoxPattCurr.ItemIndex];
    { Write new values into ListBoxes etc. }
    PattComboBoxPattCurrChange(Self);
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message , mtError, [mbOK], 0);
    end;
  end;
end;

procedure TSpeechRecogForm.PattBitBtnNewPatternSetClick(Sender: TObject);
begin
  with SpeechRecogBackPropNet.PatternSet do begin
    { Delete all existing patterns }
    DeleteAll;
    PattStringList.Clear; // Clear list containing WaveFileNames
    { Set new PatternSet }
    InputNo := 720;
    OutputNo := 8;
    { Delete number of patterns in ComboBox }
    PattComboBoxPattCurr.Clear;
    { Setting panel showing total number of patterns }
    PattPanelNrPatt.Caption := IntToStr(PatternNo);
    { Set dimension in PattPanelPatternSetDimensionInp/Outp }
    PattPanelPatternSetDimInp.Caption := IntToStr(InputNo);
    PattPanelPatternSetDimTarg.Caption := IntToStr(OutputNo);
    { Reset TargetVector panel }
    PattPanelValInpCurr.Caption := '';
    PattEditValTargCurr.Text := '';
  end;
  { Set ListBoxes }
  PattComboBoxPattCurrChange(Self);
  { Write file name in Edit }
  PattEditFilePtn.Text := 'Untitled.ptn';
end;

procedure TSpeechRecogForm.PattBitBtnOpenPatternSetClick(Sender: TObject);
var
  i: Integer;
begin
  with SpeechRecogNetFileDialog do try
    { Preset parameters for OpenDialog }
    TemplateName := 'NetDialog';
    DialogMode := dmOpen;
    Title := 'ffnen einer MusterSetDatei';
    { Filter on Pattern }
    Filter := DefaultFilter;
    FilterIndex := 5;
    FileName := PattEditFilePtn.Text;
    DefaultExt := 'ptn';
    if (Execute = True) then begin
      SpeechRecogBackPropNet.LoadFromFile(FileName);
      { Write file name in Edit }
      PattEditFilePtn.Text := ExtractFileName(FileName);
    end;
    { Delete number of patterns in ComboBox }
    PattComboBoxPattCurr.Clear;
    with SpeechRecogBackPropNet.PatternSet do begin
      { Write new pattern number into ComboBox }
      for i := 0 to (PatternNo - 1) do begin
        PattComboBoxPattCurr.Items.Add(IntToStr(PattComboBoxPattCurr.Items.Count + 1));
        PattStringList.Add('Untitled');
      end;
      PattComboBoxPattCurr.ItemIndex := (PattComboBoxPattCurr.Items.Count - 1);
      { Set dimension in PattPanelPatternSetDimensionInp/Outp }
      PattPanelPatternSetDimInp.Caption := IntToStr(InputNo);
      PattPanelPatternSetDimTarg.Caption := IntToStr(OutputNo);
      { Setting panel showing total number of patterns }
      PattPanelNrPatt.Caption := IntToStr(PatternNo);
    end;
    { Set ListBoxes }
    PattComboBoxPattCurrChange(Self);
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message , mtError, [mbOK], 0);
    end;
  end;
end;

procedure TSpeechRecogForm.PattBitBtnSavePatternSetClick(Sender: TObject);
begin
  with SpeechRecogNetFileDialog do try
    { Preset parameters }
    TemplateName := 'NetDialog';
    DialogMode := dmSave;
    Title := 'Speichern einer MusterSetDatei';
    { Filter on Pattern }
    Filter := DefaultFilter;
    FilterIndex := 5;
    FileName := PattEditFilePtn.Text;
    DefaultExt := 'ptn';
    if Execute then begin
      SpeechRecogBackPropNet.SaveToFile(FileName);
      PattEditFilePtn.Text := ExtractFileName(FileName);
    end;
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message , mtError, [mbOK], 0)
    end;
  end;
end;

procedure TSpeechRecogForm.PattComboBoxPattCurrChange(Sender: TObject);
var
  i: Integer;
begin
  { Set listboxes }
  PattListBoxInp.Clear;
  PattListBoxTarg.Clear;
  if (PattComboBoxPattCurr.ItemIndex >= 0) then
    with PattComboBoxPattCurr do begin
      { Setting labels showing size of input/target vector of pattern }
      PattLabelListInp.Caption := 'EingangsVektor ( ' +
        IntToStr(SpeechRecogBackPropNet.PatternSet.Items[ItemIndex].Inputs.Count) + ' )';
      PattLabelListTarg.Caption := 'ZielVektor ( ' +
        IntToStr(SpeechRecogBackPropNet.PatternSet.Items[ItemIndex].Outputs.Count) + ' )';
      { Fill ListBoxes and panel/edit if input/target vectors exist }
      if (SpeechRecogBackPropNet.PatternSet.Items[ItemIndex].Inputs.Count > 0) then begin
        for i := 0 to (SpeechRecogBackPropNet.PatternSet.Items[ItemIndex].Inputs.Count - 1) do
          PattListBoxInp.Items.Add(FloatToStr(SpeechRecogBackPropNet.PatternSet.Items[ItemIndex].Inputs[i]));
        PattPanelValInpCurr.Caption := PattStringList.Strings[ItemIndex];
      end;
      if (SpeechRecogBackPropNet.PatternSet.Items[ItemIndex].Outputs.Count > 0) then begin
        for i := 0 to (SpeechRecogBackPropNet.PatternSet.Items[ItemIndex].Outputs.Count - 1) do
          PattListBoxTarg.Items.Add(FloatToStr(SpeechRecogBackPropNet.PatternSet.Items[ItemIndex].Outputs[i]));
        PattListBoxTarg.ItemIndex := 0;
      end;
    end
  else begin
    { Setting labels showing emptiness of input/target vectors }
    PattLabelListInp.Caption := 'EingangsVektor';
    PattLabelListTarg.Caption := 'ZielVektor';
    { Set ListBoxes to empty }
    PattListBoxTarg.ItemIndex := -1;
  end;
end;

procedure TSpeechRecogForm.PattEditValTargCurrExit(Sender: TObject);
begin
  with PattEditValTargCurr do try
    SpeechRecogBackPropNet.PatternSet.Items[PattComboBoxPattCurr.ItemIndex].
      Outputs[PattListBoxTarg.ItemIndex] := StrToFloat(Text);
    PattComboBoxPattCurrChange(Self);
  except
    on EConvertError do begin
      MessageBeep(0);
      MessageDlg('Wert fr Zielvektor prfen!', mtError, [mbOk], 0);
      Text := FloatToStr(SpeechRecogBackPropNet.PatternSet.
        Items[PattComboBoxPattCurr.ItemIndex].Outputs[PattListBoxTarg.ItemIndex]);
      SetFocus;
    end;
  end;
end;

procedure TSpeechRecogForm.PattEditValTargCurrKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then PattEditValTargCurrExit(Self);
end;

procedure TSpeechRecogForm.PattListBoxTargExit(Sender: TObject);
begin
  with PattListBoxTarg do
    PattEditValTargCurr.Text := Items[ItemIndex];
  PattEditValTargCurr.SetFocus;
end;

procedure TSpeechRecogForm.PattListBoxTargKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Return) then PattListBoxTargExit(Self);
end;

procedure TSpeechRecogForm.PattSpeedButtonNeuroNetClick(Sender: TObject);
var
  LibHandle: THandle;
  CallBackPropagation: TCallBackPropagation;
  ExeFilePath: string;
begin
  try
    LibHandle := LoadLibrary('BackPropagation.dll');
    if (LibHandle < 32) then begin
      MessageBeep(0);
      raise EDLLLoadError.Create('Kann DLL "BackPropagation.dll" ' +
        'fr Formular "Fehlerrckfhrungsnetz" nicht laden!');
    end;
    { Get address of dll }
    @CallBackPropagation := GetProcAddress(LibHandle, 'CallBackPropagation');
    { Set status in SpeechRecogForm }
    SpeechRecogSetStatus(3, 'Aktives Netz: Fehlerrckfhrung');
    SpeechRecogStatusBar.Refresh;
    { Call modal form BackPropForm }
    CallBackPropagation;
    { Free DLL }
    FreeLibrary(LibHandle);
    { Set status in SpeechRecogForm }
    SpeechRecogSetStatus(3, 'Warte auf Eingabe');
    SpeechRecogStatusBar.Refresh;
    { Get path of application }
    ExeFilePath := ExtractFilePath(Application.ExeName);
    { Set hints and HelpFile back to those of BasicForm }
    Application.OnHint := SpeechRecogStatusBarShowHint;
    Application.HelpFile := ExeFilePath + 'Help\SpeRecog.hlp';
  except
    on E: EDLLLoadError do begin
      MessageBeep(0);
      MessageDlg(E.Message, mtInformation, [mbOk], 0);
    end;
  end;
end;

{ TabSheet 'Recall' }

procedure TSpeechRecogForm.RecallActionDefeat;
begin
  RecallResetImages;
  with SpeechRecogTimer do begin
    OnTimer := SpeechRecogTimerDefeatTimer;
    Interval := 100;
    Enabled := True;
  end;
  with RecallImagePrince do begin
    Invalidate;
    SpeechRecogPrinceImageList.Draw(Canvas, 0, 0, 2);
  end;
  with RecallImageFlower do begin
    Invalidate;
    SpeechRecogFlowerImageList.Draw(Canvas, 0, 0, 2);
  end;
end;

procedure TSpeechRecogForm.RecallActionStandard;
begin
  RecallResetImages;
  with RecallImagePrince do begin
    Invalidate;
    SpeechRecogPrinceImageList.Draw(Canvas, 0, 0, 0);
  end;
  with RecallImageFlower do begin
    Invalidate;
    SpeechRecogFlowerImageList.Draw(Canvas, 0, 0, 0);
  end;
  with RecallShape3 do begin
    Top := 170;
    RecallShape3.Brush.Color := $002794ED;
  end;
  RecallPanelResult.Caption := '';
end;

procedure TSpeechRecogForm.RecallActionSuccess;
begin
  RecallResetImages;
  with SpeechRecogTimer do begin
    OnTimer := SpeechRecogTimerSuccessTimer;
    Interval := 100;
    Enabled := True;
  end;
  with RecallImagePrince do begin
    Invalidate;
    SpeechRecogPrinceImageList.Draw(Canvas, 0, 0, 1);
  end;
  with RecallImageFlower do begin
    Invalidate;
    SpeechRecogFlowerImageList.Draw(Canvas, 0, 0, 1);
  end;
end;

procedure TSpeechRecogForm.RecallBitBtnCfgClick(Sender: TObject);
begin
  with SpeechRecogNetFileDialog do try
    TemplateName := 'NetDialog';
    DialogMode := dmOpen;
    Title := 'ffnen einer KonfigurationsDatei';
    { Filter on Config }
    Filter := DefaultFilter;
    FilterIndex := 2;
    FileName := RecallPanelCfg.Caption;
    DefaultExt := 'cfg';
    if (Execute = True) then begin
      SpeechRecogBackPropNet.LoadFromFile(FileName);
      RecallPanelCfg.Caption := ExtractFileName(FileName);
    end;
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message , mtError, [mbOK], 0)
    end;
  end;
end;

procedure TSpeechRecogForm.RecallBitBtnLrnClick(Sender: TObject);
begin
  with SpeechRecogNetFileDialog do try
    TemplateName := 'NetDialog';
    DialogMode := dmOpen;
    Title := 'ffnen einer LernDatenDatei';
    { Filter on Learn }
    Filter := DefaultFilter;
    FilterIndex := 3;
    FileName := RecallPanelLrn.Caption;
    DefaultExt := 'lrn';
    if (Execute = True) then begin
      SpeechRecogBackPropNet.LoadFromFile(FileName);
      RecallPanelLrn.Caption := ExtractFileName(FileName);
    end;
  except
    on E: Exception do begin
      MessageBeep(0);
      MessageDlg('Ein Fehler ist aufgetreten:' + #13#10 + E.Message , mtError, [mbOK], 0);
    end;
  end;
end;

procedure TSpeechRecogForm.RecallBitBtnOfflineClick(Sender: TObject);
begin
  { Check }
  if ((FileWaveFile = nil) or (RecallPanelLrn.Caption = 'Keine')) then begin
    MessageBeep(0);
    raise Exception.Create('Netz nicht angelernt oder keine Audio-Datei!');
  end;
  { Call procedure starting recall, checking result, raising object activity }
  RecallResultCheck;
end;

procedure TSpeechRecogForm.RecallBitBtnOnlineClick(Sender: TObject);
var
  Dummy: Boolean;
begin
  { Check }
  if (RecallPanelLrn.Caption = 'Keine') then begin
    MessageBeep(0);
    raise Exception.Create('Netz nicht angelernt!');
  end;
  { MediaPlayer: record & save file, then load data and calculate parameters }
  FileMediaPlayerClick(Self, btRecord, Dummy);
end;

procedure TSpeechRecogForm.RecallBitBtnResetClick(Sender: TObject);
var
  i: Integer;
begin
  with SpeechRecogBackPropNet do begin
    { Set net configuration to initial conditions }
    InputNo := 720;
    LayerNo := 2;
    for i := 0 to (LayerNo - 1) do
      case i of
        0: with Items[i] do begin
          ActFunction := faLinear;
          NeuronNo := 8;
          OutpFunction := foArcTan;
          OutputMax := 1.2;
          OutputMin := -0.2;
          PropFunction := fpScalProd;
          Scale := 1;
          Slope := 1;
        end;
        1: with Items[i] do begin
          ActFunction := faLinear;
          NeuronNo := 8;
          OutpFunction := foArcTan;
          OutputMax := 1.2;
          OutputMin := -0.2;
          PropFunction := fpScalProd;
          Scale := 1;
          Slope := 1;
        end;
      end;
    { Reset all learn parameters to random values }
    ResetAll;
    { Write initial values into net component }
    AdaptConst := 0;
    AdaptConstFactor := 1; // AdaptConstStep := 0
    CycleLimit := 10000;
    ErrorLimit := 0.01;
    Interval := 1;
    LearnMethod := lmEpochal;
    LearnParams := [lpWeights,lpBias];
    LearnRate := 0.01;
    LearnRateDec := 1.0;
    LearnRateInc := 1.0;
    Momentum := 0.3;
    ParamsMax := 0.9;
    ParamsMin := -0.9;
  end;
  { Write FileNames in Panels }
  RecallPanelCfg.Caption := 'Vorgabe (720-8-8)';
  RecallPanelLrn.Caption := 'Keine';
end;

procedure TSpeechRecogForm.RecallBitBtnWavClick(Sender: TObject);
begin
  FileBitBtnOpenClick(Self);
end;

procedure TSpeechRecogForm.RecallImagePrinceClick(Sender: TObject);
begin
  RecallActionStandard;
end;

procedure TSpeechRecogForm.RecallPanelActionClick(Sender: TObject);
begin
  RecallActionStandard;
end;

procedure TSpeechRecogForm.RecallRadioGroupTypeClick(Sender: TObject);
begin
  if (RecallRadioGroupType.ItemIndex = 0) then begin
    RecallBitBtnOffline.Enabled := True;
    RecallBitBtnOnline.Enabled := False;
    RecallBitBtnWav.Enabled := True;
    RecallPanelWav.Enabled := True;
  end;
  if (RecallRadioGroupType.ItemIndex = 1) then begin
    RecallBitBtnOffline.Enabled := False;
    RecallBitBtnOnline.Enabled := True;
    RecallBitBtnWav.Enabled := False;
    RecallPanelWav.Enabled := False;
  end;
end;

procedure TSpeechRecogForm.RecallResetImages;
var
  ExeFilePath: string;
  Picture: TPicture;
begin
  { Get path of Application }
  ExeFilePath := ExtractFilePath(Application.ExeName);
  try
    Picture := TPicture.Create;
    Picture.Bitmap.LoadFromResourceName(HInstance, 'PrinceOverlay');
    with RecallImagePrince.Canvas do begin
      Draw(0, 0, TGraphic(Picture.Bitmap));
      Brush.Color := $00420000;
      FloodFill(10, 10, clWhite, fsBorder);
      Brush.Color := $00C6F7FF;
      FloodFill(10, 190, $00420000, fsBorder);
    end;
    Picture.Bitmap.LoadFromResourceName(HInstance, 'FlowerOverlay');
    with RecallImageFlower.Canvas do begin
      Draw(0, 0, TGraphic(Picture.Bitmap));
      Brush.Color := $00C6F7FF;
      FloodFill(10, 10, $00420000, fsBorder);
    end;
    Picture.Free;
  except
    on EInvalidGraphic do begin
      MessageBeep(0);
      MessageDlg('Kein Graphik-Datei-Format!', mtWarning, [mbOk], 0);
    end;
  end;
end;

procedure TSpeechRecogForm.RecallResultCheck;

  { Involve Base to the Exponent }
  function Involve(const Base: Double; const Exponent: Double): Double;
  begin
    try
      Result := Exp(Ln(Base) * Exponent);
    except
      on Exception do Result := 0;
    end;
  end;

var
  CheckSum: Integer;
  Defeat: Boolean;
  i, j, k, n: Integer;
  Result: Integer;
  ResultString: string;
  ResultValue: Integer;
  ResultWord: string;
begin
  with SpeechRecogBackPropNet do begin
    { Set WaitCursor }
    Screen.Cursor := crSpeechRecogWaitCursor;
    { Set status }
    SpeechRecogSetStatus(2, 'Reproduziere Neuronales Netz');
    SpeechRecogStatusBar.Refresh;
    { Set RecallSet }
    RecallSet.DeleteAll;
    RecallSet.PatternNo := 1;
    RecallSet.InputNo := InputNo;
    RecallSet.OutputNo := Items[(LayerNo - 1)].NeuronNo;
    if (FileWaveFile = nil) then begin
      MessageBeep(0);
      raise Exception.Create('Keine Audio-Datei geladen!');
    end;
    { Hand over signal processing results to PatternSet of BackPropNet }
    with FileWaveFile do begin
      for j := 0 to (DataLoud.Count - 1) do begin
        Result := 0;
        for k := 0 to (39 - 1) do
          if ((DataLoud[j] > (k * 50)) and (DataLoud[j] <= ((k + 1) * 50))) then Result := k;
        k := 39;
        if (DataLoud[j] > (k * 50)) then Result := k;
        for n := 0 to 39 do
          if (n <= Result) then
            RecallSet.Items[0].Inputs[(j * 40) + n] := 1;
      end;
    end;
    { Set procedures for events within net component }
    OnExit := nil;
    OnInterval := nil;
    { Start recall }
    Recall;
    { Set status }
    SpeechRecogSetStatus(1, 'Checke Ergebnis');
    SpeechRecogStatusBar.Refresh;
    { Get letters of result string out of RecallSet OutputVector }
    Defeat := False;
    ResultValue := 0;
    ResultString := '';
    CheckSum := 0;
    for i := 0 to (RecallSet.Items[0].Outputs.Count - 1) do begin
      if (Round(RecallSet.Items[0].Outputs[i]) < 0) or
        (Round(RecallSet.Items[0].Outputs[i]) > 1) then Defeat := True;
      if (Round(RecallSet.Items[0].Outputs[i]) = 1) then ResultValue := (i + 1);
      ResultString := (ResultString + IntToStr(Round(RecallSet.Items[0].Outputs[i])));
      CheckSum := (CheckSum + Round(RecallSet.Items[0].Outputs[i]));
    end;
    case ResultValue of
      1: ResultWord := 'Wind';
      2: ResultWord := 'Star';
      3: ResultWord := 'Rechts';
      4: ResultWord := 'Radio';
      5: ResultWord := 'Herbst';
      6: ResultWord := 'Elf';
      7: ResultWord := 'Dickicht';
      8: ResultWord := 'Acht';
    end;
    ResultString := (ResultString + '   -->   ' + ResultWord);
    { Write result in PanelResult }
    RecallPanelResult.Caption := ResultString;
    { Check for detection and errors }
    if ((CheckSum > 1) or (CheckSum = 0)) then Defeat := True;
  end;
  { SpeechRecognition successful? }
  if (Defeat = True) then RecallActionDefeat else RecallActionSuccess;
  { Set status }
  SpeechRecogSetStatus(0, 'Warte auf Eingabe');
  SpeechRecogStatusBar.Refresh;
  { Reset WaitCursor }
  Screen.Cursor := crDefault;
end;

procedure TSpeechRecogForm.RecallShape1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  RecallActionStandard;
end;

{ Section 'Speech Recognition' }

procedure TSpeechRecogForm.SpeechRecogBackPropNetLearnExit(Sender: TBackPropNet;
  const ExitCode: TExitCode);
var
  HelpString: string;
begin
  { Finishing announcement }
  HelpString := 'Keine.';
  if (ExitCode = exBreak) then HelpString := 'Abbruch durch den Benutzer.';
  if (ExitCode = exError) then HelpString := 'Unterschreiten der angegebenen Fehlergrenze.';
  if (ExitCode = exCycle) then HelpString := 'Erreichen der vorgegebenen Lernschrittanzahl.';
  MessageBeep(0);
  MessageDlg('Lernen beendet. Ursache: ' + HelpString, mtInformation, [mbOk], 0);
end;

procedure TSpeechRecogForm.SpeechRecogTimerDefeatTimer(Sender: TObject);
begin
  if (RecallShape3.Top < 200) then RecallShape3.Top := (RecallShape3.Top + 1)
  else SpeechRecogTimer.Enabled := False;
end;

procedure TSpeechRecogForm.SpeechRecogPageControlChange(Sender: TObject);
begin
  with SpeechRecogPageControl do begin
    { Set help contexts }
    if (ActivePage = FileTabSheet) then HelpContext := 1;
    if (ActivePage = PattTabSheet) then HelpContext := 2;
    if (ActivePage = RecallTabSheet) then HelpContext := 3;
  end;
end;

procedure TSpeechRecogForm.SpeechRecogTimerRecordTimer(Sender: TObject);
var
  Dummy: Boolean;
begin
  FileMediaPlayerClick(Sender, btStop, Dummy);
end;

procedure TSpeechRecogForm.SpeechRecogSetStatus(ImageIndex: Integer; Text: string);
var
  HelpRect: TRect;
begin
  SpeechRecogStatusImageIndex := ImageIndex;
  with SpeechRecogStatusBar do begin
    with HelpRect do begin
      Left := 0;
      Top := 0;
      Right := Panels[0].Width;
      Bottom := Height;
    end;
    SpeechRecogStatusBarDrawPanel(SpeechRecogStatusBar, Panels[0], HelpRect);
    Panels[1].Text := Text;
  end;
end;

procedure TSpeechRecogForm.SpeechRecogStatusBarDrawPanel(StatusBar: TStatusBar;
  Panel: TStatusPanel; const Rect: TRect);
var
  Picture: TPicture;
begin
  Invalidate;
  { Overpaint old image }
  with StatusBar.Canvas do begin
    Brush.Color := clSilver;
    FillRect(Rect);
  end;
  { Paint new image }
  Picture := TPicture.Create;
  case SpeechRecogStatusImageIndex of
    0: Picture.Bitmap.LoadFromResourceName(HInstance, 'StatusGreen');
    1: Picture.Bitmap.LoadFromResourceName(HInstance, 'StatusYellow');
    2: Picture.Bitmap.LoadFromResourceName(HInstance, 'StatusRed');
    3: Picture.Bitmap.LoadFromResourceName(HInstance, 'StatusBackProp');
  end;
  StatusBar.Canvas.Draw(3, 3, TGraphic(Picture.Bitmap));
  Picture.Free;
end;

procedure TSpeechRecogForm.SpeechRecogStatusBarShowHint(Sender: TObject);
begin
  SpeechRecogStatusBar.Panels[2].Text := Application.Hint;
end;

procedure TSpeechRecogForm.SpeechRecogTimerSuccessTimer(Sender: TObject);
begin
  if (RecallShape3.Top > 100) then RecallShape3.Top := (RecallShape3.Top - 1) else begin
    RecallShape3.Brush.Color := clYellow;
    SpeechRecogTimer.Enabled := False;
  end;
end;

end.
